# Copyright 2015
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import sys

from tempest import config
from tempest.test_discover import plugins

from cinder_tempest_plugin import config as project_config


class CinderTempestPlugin(plugins.TempestPlugin):
    def load_tests(self):
        """Provides information to load the plugin tests.

        :return: A tuple with the first value being the test dir and the
                 second being the top level dir.
        """
        base_path = os.path.split(os.path.dirname(
            os.path.abspath(__file__)))[0]
        test_dir = "cinder_tempest_plugin"
        full_test_dir = os.path.join(base_path, test_dir)
        return full_test_dir, base_path

    def register_opts(self, conf):
        """Adds additional configuration options to tempest.

        This method will be run for the plugin during the register_opts()
        function in tempest.config

        :param conf: The conf object that can be used to register additional
                     options.
        """
        config.register_opt_group(conf, config.volume_feature_group,
                                  project_config.cinder_option)

        # Define the 'barbican' service_available option, but only if the
        # barbican_tempest_plugin isn't present. It also defines the option,
        # and we need to avoid a duplicate option registration.
        if 'barbican_tempest_plugin' not in sys.modules:
            config.register_opt_group(conf, config.service_available_group,
                                      project_config.barbican_service_option)

    def get_opt_lists(self):
        """Get a list of options for sample config generation.

        :return: A list of tuples with the group name and options in that
                 group.
        """
        opt_lists = [
            (config.volume_feature_group.name, project_config.cinder_option),
        ]

        if 'barbican_tempest_plugin' not in sys.modules:
            opt_lists.append((config.service_available_group.name,
                              project_config.barbican_service_option))

        return opt_lists

    def get_service_clients(self):
        volumes_config = config.service_client_config('volume')

        consistencygroups_params = {
            'name': 'consistencygroups_v3',
            'service_version': 'consistencygroups.v3',
            'module_path': 'cinder_tempest_plugin.services.'
                           'consistencygroups_client',
            'client_names': ['ConsistencyGroupsClient'],
        }
        consistencygroups_params.update(volumes_config)

        volumerevert_params = {
            'name': 'volume_revert_v3',
            'service_version': 'volume_revert.v3',
            'module_path': 'cinder_tempest_plugin.services.'
                           'volume_revert_client',
            'client_names': ['VolumeRevertClient'],
        }
        volumerevert_params.update(volumes_config)

        return [consistencygroups_params, volumerevert_params]



# Copyright 2017 NEC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import io

from tempest.common import compute
from tempest.common import waiters
from tempest import config
from tempest.lib.common import api_version_utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest import test

CONF = config.CONF


class BaseVolumeTest(api_version_utils.BaseMicroversionTest,
                     test.BaseTestCase):
    """Base test case class for all Cinder API tests."""

    credentials = ['primary']

    @classmethod
    def skip_checks(cls):
        super(BaseVolumeTest, cls).skip_checks()

        if not CONF.service_available.cinder:
            skip_msg = ("%s skipped as Cinder is not available" % cls.__name__)
            raise cls.skipException(skip_msg)

        api_version_utils.check_skip_with_microversion(
            cls.min_microversion, cls.max_microversion,
            CONF.volume.min_microversion, CONF.volume.max_microversion)

    @classmethod
    def setup_clients(cls):
        super(BaseVolumeTest, cls).setup_clients()
        cls.backups_client = cls.os_primary.backups_client_latest
        cls.volumes_client = cls.os_primary.volumes_client_latest
        cls.snapshots_client = cls.os_primary.snapshots_client_latest
        cls.volume_revert_client = (
            cls.os_primary.volume_revert_v3.VolumeRevertClient()
        )

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources()
        super(BaseVolumeTest, cls).setup_credentials()

    def setUp(self):
        super(BaseVolumeTest, self).setUp()

    @classmethod
    def resource_setup(cls):
        super(BaseVolumeTest, cls).resource_setup()
        cls.request_microversion = (
            api_version_utils.select_request_microversion(
                cls.min_microversion,
                CONF.volume.min_microversion))
        cls.setup_api_microversion_fixture(
            volume_microversion=cls.request_microversion)

    @classmethod
    def create_volume(cls, wait_until='available', **kwargs):
        """Wrapper utility that returns a test volume.

           :param wait_until: wait till volume status, None means no wait.
        """
        if 'size' not in kwargs:
            kwargs['size'] = CONF.volume.volume_size

        if 'imageRef' in kwargs:
            image = cls.os_primary.image_client_v2.show_image(
                kwargs['imageRef'])
            min_disk = image['min_disk']
            kwargs['size'] = max(kwargs['size'], min_disk)

        if 'name' not in kwargs:
            name = data_utils.rand_name(cls.__name__ + '-Volume')
            kwargs['name'] = name

        if CONF.compute.compute_volume_common_az:
            kwargs.setdefault('availability_zone',
                              CONF.compute.compute_volume_common_az)

        volume = cls.volumes_client.create_volume(**kwargs)['volume']
        cls.addClassResourceCleanup(
            cls.volumes_client.wait_for_resource_deletion, volume['id'])
        cls.addClassResourceCleanup(test_utils.call_and_ignore_notfound_exc,
                                    cls.volumes_client.delete_volume,
                                    volume['id'])
        if wait_until:
            waiters.wait_for_volume_resource_status(cls.volumes_client,
                                                    volume['id'], wait_until)
        return volume

    @classmethod
    def create_snapshot(cls, volume_id=1, **kwargs):
        """Wrapper utility that returns a test snapshot."""
        if 'name' not in kwargs:
            name = data_utils.rand_name(cls.__name__ + '-Snapshot')
            kwargs['name'] = name

        snapshot = cls.snapshots_client.create_snapshot(
            volume_id=volume_id, **kwargs)['snapshot']
        cls.addClassResourceCleanup(
            cls.snapshots_client.wait_for_resource_deletion, snapshot['id'])
        cls.addClassResourceCleanup(test_utils.call_and_ignore_notfound_exc,
                                    cls.snapshots_client.delete_snapshot,
                                    snapshot['id'])
        waiters.wait_for_volume_resource_status(cls.snapshots_client,
                                                snapshot['id'], 'available')
        return snapshot

    def create_backup(self, volume_id, backup_client=None, **kwargs):
        """Wrapper utility that returns a test backup."""
        if backup_client is None:
            backup_client = self.backups_client
        if 'name' not in kwargs:
            name = data_utils.rand_name(self.__class__.__name__ + '-Backup')
            kwargs['name'] = name

        backup = backup_client.create_backup(
            volume_id=volume_id, **kwargs)['backup']
        self.addCleanup(backup_client.wait_for_resource_deletion, backup['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        backup_client.delete_backup, backup['id'])
        waiters.wait_for_volume_resource_status(backup_client, backup['id'],
                                                'available')
        return backup

    def create_server(self, wait_until='ACTIVE', **kwargs):
        name = kwargs.pop(
            'name',
            data_utils.rand_name(self.__class__.__name__ + '-instance'))

        if wait_until == 'SSHABLE' and not kwargs.get('validation_resources'):
            kwargs['validation_resources'] = (
                self.get_test_validation_resources(self.os_primary))
            kwargs['validatable'] = True

        tenant_network = self.get_tenant_network()
        body, _ = compute.create_test_server(
            self.os_primary,
            tenant_network=tenant_network,
            name=name,
            wait_until=wait_until,
            **kwargs)

        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        waiters.wait_for_server_termination,
                        self.os_primary.servers_client, body['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.os_primary.servers_client.delete_server,
                        body['id'])
        return body

    @classmethod
    def create_image_with_data(cls, **kwargs):
        # we do this as a class method so we can use the
        # addClassResourceCleanup functionality of tempest.test.BaseTestCase
        images_client = cls.os_primary.image_client_v2
        if 'min_disk' not in kwargs:
            kwargs['min_disk'] = 1
        response = images_client.create_image(**kwargs)
        image_id = response['id']
        cls.addClassResourceCleanup(
            images_client.wait_for_resource_deletion, image_id)
        cls.addClassResourceCleanup(
            test_utils.call_and_ignore_notfound_exc,
            images_client.delete_image, image_id)

        # upload "data" to image
        image_file = io.BytesIO(data_utils.random_bytes(size=1024))
        images_client.store_image_file(image_id, image_file)

        waiters.wait_for_image_status(images_client, image_id, 'active')
        image = images_client.show_image(image_id)
        return image


class BaseVolumeAdminTest(BaseVolumeTest):
    """Base test case class for all Volume Admin API tests."""

    credentials = ['primary', 'admin']

    @classmethod
    def setup_clients(cls):
        super(BaseVolumeAdminTest, cls).setup_clients()

        cls.admin_volume_types_client = cls.os_admin.volume_types_client_latest
        cls.admin_backups_client = cls.os_admin.backups_client_latest
        cls.admin_volume_client = cls.os_admin.volumes_client_latest
        cls.admin_consistencygroups_client = (
            cls.os_admin.consistencygroups_v3.ConsistencyGroupsClient()
        )

    @classmethod
    def create_volume_type(cls, name=None, **kwargs):
        """Create a test volume-type"""

        name = name or data_utils.rand_name(cls.__name__ + '-volume-type')
        volume_type = cls.admin_volume_types_client.create_volume_type(
            name=name, **kwargs)['volume_type']
        cls.addClassResourceCleanup(cls._clear_volume_type, volume_type)
        return volume_type

    @classmethod
    def _clear_volume_type(cls, volume_type):
        # If image caching is enabled, we must delete the cached volume
        # before cinder will allow us to delete the volume_type.  This function
        # solves that problem by taking the brute-force approach of deleting
        # any volumes of this volume_type that exist *no matter what project
        # they are in*.  Since this won't happen until the teardown of the
        # test class, that should be OK.
        type_id = volume_type['id']
        type_name = volume_type['name']

        volumes = cls.admin_volume_client.list_volumes(
            detail=True, params={'all_tenants': 1})['volumes']
        for volume in [v for v in volumes if v['volume_type'] == type_name]:
            test_utils.call_and_ignore_notfound_exc(
                cls.admin_volume_client.delete_volume, volume['id'])
            cls.admin_volume_client.wait_for_resource_deletion(volume['id'])

        test_utils.call_and_ignore_notfound_exc(
            cls.admin_volume_types_client.delete_volume_type, type_id)
        test_utils.call_and_ignore_notfound_exc(
            cls.admin_volume_types_client.wait_for_resource_deletion, type_id)


class CreateMultipleResourceTest(BaseVolumeTest):

    def _create_multiple_resource(self, callback, repeat_count=5,
                                  **kwargs):

        res = []
        for _ in range(repeat_count):
            res.append(callback(**kwargs)['id'])
        return res

    def _wait_for_multiple_resources(self, callback, wait_list, **kwargs):

        for r in wait_list:
            callback(resource_id=r, **kwargs)



# Copyright 2020 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators

import testtools

from cinder_tempest_plugin.scenario import manager

CONF = config.CONF


class SnapshotDataIntegrityTests(manager.ScenarioTest):

    def setUp(self):
        super(SnapshotDataIntegrityTests, self).setUp()
        self.validation_resources = self.get_test_validation_resources(
            self.os_primary)
        # NOTE(danms): If validation is enabled, we will have a keypair to use,
        # otherwise we need to create our own.
        if 'keypair' in self.validation_resources:
            self.keypair = self.validation_resources['keypair']
        else:
            self.keypair = self.create_keypair()
        self.security_group = self.create_security_group()

    @decorators.idempotent_id('ff10644e-5a70-4a9f-9801-8204bb81fb61')
    @utils.services('compute', 'volume', 'image', 'network')
    def test_snapshot_data_integrity(self):
        """This test checks the data integrity after creating and restoring

        snapshots. The procedure is as follows:

        1) Create an instance with ephemeral disk
        2) Create a volume, attach it to the instance and create a filesystem
           on it and mount it
        3) Create a file and write data into it, Unmount it
        4) create snapshot
        5) repeat 3 and 4 two more times (simply creating 3 snapshots)

        Now create volume from the snapshots one by one, attach it to the
        instance and check the number of files and file content at each
        point when snapshot was created.
        """

        # Create an instance
        server = self.create_server(
            key_name=self.keypair['name'],
            validatable=True,
            validation_resources=self.validation_resources,
            wait_until='SSHABLE',
            security_groups=[{'name': self.security_group['name']}])

        # Create an empty volume
        volume = self.create_volume()

        instance_ip = self.get_server_ip(server)

        # Attach volume to instance and find it's device name (eg: /dev/vdb)
        volume_device_name, __ = self._attach_and_get_volume_device_name(
            server, volume, instance_ip, self.keypair['private_key'])

        # Create filesystem on the volume
        self._make_fs(instance_ip, self.keypair['private_key'], server,
                      volume_device_name)

        # Write data to volume
        file1_md5 = self.create_md5_new_file(
            instance_ip, dev_name=volume_device_name, filename="file1",
            private_key=self.keypair['private_key'],
            server=instance_ip)

        # Create first snapshot
        snapshot1 = self.create_volume_snapshot(volume['id'], force=True)

        # Write data to volume
        file2_md5 = self.create_md5_new_file(
            instance_ip, dev_name=volume_device_name, filename="file2",
            private_key=self.keypair['private_key'],
            server=instance_ip)

        # Create second snapshot
        snapshot2 = self.create_volume_snapshot(volume['id'], force=True)

        # Write data to volume
        file3_md5 = self.create_md5_new_file(
            instance_ip, dev_name=volume_device_name, filename="file3",
            private_key=self.keypair['private_key'],
            server=instance_ip)

        # Create third snapshot
        snapshot3 = self.create_volume_snapshot(volume['id'], force=True)

        # Detach the volume
        self.nova_volume_detach(server, volume)

        snap_map = {1: snapshot1, 2: snapshot2, 3: snapshot3}
        file_map = {1: file1_md5, 2: file2_md5, 3: file3_md5}

        # Loop over 3 times to check the data integrity of all 3 snapshots
        for i in range(1, 4):
            # Create volume from snapshot, attach it to instance and check file
            # and contents for snap
            volume_snap = self.create_volume(snapshot_id=snap_map[i]['id'])
            volume_device_name, __ = self._attach_and_get_volume_device_name(
                server, volume_snap, instance_ip, self.keypair['private_key'])
            count_snap, md5_file = self.get_md5_from_file(
                server, instance_ip, 'file' + str(i),
                dev_name=volume_device_name)
            # Detach the volume
            self.nova_volume_detach(server, volume_snap)

            self.assertEqual(count_snap, i)
            self.assertEqual(file_map[i], md5_file)


class SnapshotDependencyTests(manager.ScenarioTest):
    @testtools.skipUnless(
        CONF.volume_feature_enabled.enable_volume_image_dep_tests,
        'dependency tests not enabled')
    @decorators.idempotent_id('e7028f52-f6d4-479c-8809-6f6cf96cfe0f')
    @utils.services('image', 'volume')
    def test_snapshot_removal(self):
        volume_1 = self.create_volume()

        snapshot_1 = self.create_volume_snapshot(volume_1['id'], force=True)
        waiters.wait_for_volume_resource_status(
            self.snapshots_client, snapshot_1['id'], 'available')

        clone_kwargs = {'snapshot_id': snapshot_1['id'],
                        'size': volume_1['size']}
        volume_2 = self.volumes_client.create_volume(**clone_kwargs)['volume']

        waiters.wait_for_volume_resource_status(
            self.volumes_client, volume_2['id'], 'available')
        volume_2 = self.volumes_client.show_volume(volume_2['id'])['volume']

        self.snapshots_client.delete_snapshot(snapshot_1['id'])
        self.snapshots_client.wait_for_resource_deletion(snapshot_1['id'])

        self.volumes_client.delete_volume(volume_1['id'])
        self.volumes_client.wait_for_resource_deletion(volume_1['id'])

        self.volumes_client.delete_volume(volume_2['id'])
        self.volumes_client.wait_for_resource_deletion(volume_2['id'])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions

from cinder_tempest_plugin.rbac.v3 import base as rbac_base

CONF = config.CONF


class VolumeV3RbacVolumesTests(rbac_base.VolumeV3RbacBaseTests):

    min_microversion = '3.12'

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.vol_other_client = cls.os_project_admin.volumes_client_latest

    def _create_volume(self, expected_status, **kwargs):
        """Test create_volume operation.

        Args:
            expected_status: The expected HTTP response code
        """
        kwargs['size'] = CONF.volume.volume_size
        self.do_request(
            method='create_volume', expected_status=expected_status, **kwargs
        )

    def _show_volume(self, expected_status):
        """Test show_volume operation

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        self.do_request(
            method='show_volume', volume_id=volume_id,
            expected_status=expected_status
        )

    def _list_volumes(self, expected_status):
        """Test list_volumes operation

        Args:
            expected_status: The expected HTTP response code
        """
        self.create_volume(client=self.vol_other_client)
        self.do_request(method='list_volumes', expected_status=expected_status)

    def _list_volumes_detail(self, expected_status):
        """Test list_volumes details operation

        Args:
            expected_status: The expected HTTP response code
        """
        self.create_volume(client=self.vol_other_client)
        self.do_request(
            method='list_volumes', detail=True, expected_status=expected_status
        )

    def _show_volume_summary(self, expected_status):
        """Test show_volume_summary operation

        Args:
            expected_status: The expected HTTP response code
        """
        self.create_volume(client=self.vol_other_client)
        self.do_request(
            method='show_volume_summary', expected_status=expected_status
        )

    def _update_volume(self, expected_status):
        """Test update_volume operation.

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        new_desc = self.__name__ + '-update_test'
        self.do_request(
            method='update_volume', volume_id=volume_id, description=new_desc,
            expected_status=expected_status
        )

    def _set_bootable_volume(self, expected_status):
        """Test set_bootable_volume operation.

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        self.do_request(
            method='set_bootable_volume', volume_id=volume_id,
            bootable=True, expected_status=expected_status
        )

    def _delete_volume(self, expected_status):
        """Test delete_volume operation.

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        self.do_request(
            method='delete_volume', volume_id=volume_id,
            expected_status=expected_status
        )


class ProjectReaderTests(VolumeV3RbacVolumesTests):

    credentials = ['project_reader', 'project_admin']

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_reader.volumes_client_latest

    @decorators.skip_because(bug="2020113")
    @decorators.idempotent_id('3d87f960-6210-45f5-b70b-679d67a4e17e')
    def test_create_volume(self):
        self._create_volume(expected_status=exceptions.Forbidden)

    @decorators.idempotent_id('9b2667f2-744e-4d1f-8c39-17060010f19f')
    def test_show_volume(self):
        self._show_volume(expected_status=200)

    @decorators.idempotent_id('2f4da8f9-cdc5-4a6e-9143-8237634a629c')
    def test_list_volumes(self):
        self._list_volumes(expected_status=200)

    @decorators.idempotent_id('b11e59cd-d1dd-43e4-9676-22ab394f5d18')
    def test_list_volumes_detail(self):
        self._list_volumes_detail(expected_status=200)

    @decorators.idempotent_id('ef347930-54dc-432f-b742-0a060fc37ae8')
    def test_show_volume_summary(self):
        self._show_volume_summary(expected_status=200)

    @decorators.skip_because(bug="2020113")
    @decorators.idempotent_id('cda92972-7213-4fa0-bc14-ab012dc95931')
    def test_update_volume(self):
        self._update_volume(expected_status=exceptions.Forbidden)

    @decorators.skip_because(bug="2020113")
    @decorators.idempotent_id('9970b57d-8d5d-460e-931b-28a112df81e0')
    def test_set_bootable_volume(self):
        self._set_bootable_volume(expected_status=exceptions.Forbidden)

    @decorators.skip_because(bug="2020113")
    @decorators.idempotent_id('4fd4dce8-ed8a-4f05-8aac-da99858b563d')
    def test_delete_volume(self):
        self._delete_volume(expected_status=exceptions.Forbidden)



# Copyright 2022 Red Hat, Inc.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators
from tempest.scenario import manager

CONF = config.CONF


class TransferEncryptedVolumeTest(manager.EncryptionScenarioTest):

    volume_min_microversion = '3.70'
    volume_max_microversion = 'latest'

    credentials = ['primary', 'alt', 'admin']

    @classmethod
    def setup_clients(cls):
        super(TransferEncryptedVolumeTest, cls).setup_clients()

        # We need the "mv355" volume transfers client
        cls.client = cls.os_primary.volume_transfers_mv355_client_latest
        cls.alt_client = cls.os_alt.volume_transfers_mv355_client_latest
        cls.alt_volumes_client = cls.os_alt.volumes_client_latest

    @classmethod
    def skip_checks(cls):
        super(TransferEncryptedVolumeTest, cls).skip_checks()
        if not CONF.compute_feature_enabled.attach_encrypted_volume:
            raise cls.skipException('Encrypted volume attach is not supported')
        if not CONF.service_available.barbican:
            raise cls.skipException('Barbican is required')

    def setUp(self):
        super(TransferEncryptedVolumeTest, self).setUp()
        self.keypair = self.create_keypair()
        self.security_group = self.create_security_group()

    def _create_encrypted_volume_from_image(self):
        volume_type = self.create_volume_type()
        self.create_encryption_type(type_id=volume_type['id'],
                                    provider='luks',
                                    key_size=256,
                                    cipher='aes-xts-plain64',
                                    control_location='front-end')
        return self.create_volume_from_image(volume_type=volume_type['id'])

    def _create_or_get_timestamp(self, volume, timestamp_fn):
        server = self.boot_instance_from_resource(
            source_id=volume['id'],
            source_type='volume',
            keypair=self.keypair,
            security_group=self.security_group)
        server_ip = self.get_server_ip(server)
        timestamp = timestamp_fn(server_ip,
                                 private_key=self.keypair['private_key'],
                                 server=server)
        self.servers_client.delete_server(server['id'])
        waiters.wait_for_server_termination(self.servers_client, server['id'])
        return timestamp

    def _create_transfer(self, volume, transfer_client, volumes_client):
        body = transfer_client.create_volume_transfer(volume_id=volume['id'])
        transfer = body['transfer']
        waiters.wait_for_volume_resource_status(volumes_client,
                                                volume['id'],
                                                'awaiting-transfer')
        return transfer

    def _accept_transfer(self, transfer, transfer_client, volumes_client):
        _ = transfer_client.accept_volume_transfer(
            transfer['id'], auth_key=transfer['auth_key'])
        waiters.wait_for_volume_resource_status(volumes_client,
                                                transfer['volume_id'],
                                                'available')

    def _delete_transfer(self, transfer, transfer_client, volumes_client):
        _ = transfer_client.delete_volume_transfer(transfer['id'])
        waiters.wait_for_volume_resource_status(volumes_client,
                                                transfer['volume_id'],
                                                'available')

    @decorators.idempotent_id('a694dc4d-d11b-45cb-b268-62e76cc1b4f4')
    @utils.services('compute', 'volume', 'image', 'network')
    def test_create_accept_volume_transfer(self):
        """Verify the ability to transfer an encrypted volume:

        * Create an encrypted volume from image
        * Boot an instance from the volume and write a timestamp
        * Transfer the volume to another project, then transfer it back
          again to the original project (see comments in the code for why
          this is done).
        * Boot annother instance from the volume and read the timestamp
        * Verify the timestamps match, and the volume has a new
          encryption_key_id.
        """

        # Create a bootable encrypted volume.
        volume = self._create_encrypted_volume_from_image()

        # Create an instance from the volume and write a timestamp.
        timestamp_1 = self._create_or_get_timestamp(volume,
                                                    self.create_timestamp)

        # Transfer the volume to another project.
        transfer = self._create_transfer(volume,
                                         self.client,
                                         self.volumes_client)
        self._accept_transfer(transfer,
                              self.alt_client,
                              self.alt_volumes_client)

        # Transfer the volume back to the original project. This is done
        # only because it's awkward in tempest to boot an instance and
        # access it (to read the timestamp) in another project without
        # setting up another security group and group rules.
        transfer = self._create_transfer(volume,
                                         self.alt_client,
                                         self.alt_volumes_client)
        self._accept_transfer(transfer, self.client, self.volumes_client)

        # Create another instance from the volume and read the timestamp.
        timestamp_2 = self._create_or_get_timestamp(volume,
                                                    self.get_timestamp)

        self.assertEqual(timestamp_1, timestamp_2)

        # Verify the volume has a new encryption_key_id.
        encryption_key_id_1 = volume['encryption_key_id']
        volume = self.volumes_client.show_volume(volume['id'])['volume']
        encryption_key_id_2 = volume['encryption_key_id']

        self.assertNotEqual(encryption_key_id_1, encryption_key_id_2)

    @decorators.idempotent_id('00c04d27-b3c6-454c-a0b4-223a195c4a89')
    @utils.services('compute', 'volume', 'image', 'network')
    def test_create_delete_volume_transfer(self):
        """Verify the ability to cancel an encrypted volume transfer:

        * Create an encrypted volume from image
        * Boot an instance from the volume and write a timestamp
        * Create and delete a volume transfer
        * Boot annother instance from the volume and read the timestamp
        * Verify the timestamps match, and the volume has a new
          encryption_key_id.
        """

        # Create a bootable encrypted volume.
        volume = self._create_encrypted_volume_from_image()

        # Create an instance from the volume and write a timestamp.
        timestamp_1 = self._create_or_get_timestamp(volume,
                                                    self.create_timestamp)

        # Create and then delete a transfer of the volume
        transfer = self._create_transfer(volume,
                                         self.client,
                                         self.volumes_client)
        self._delete_transfer(transfer, self.client, self.volumes_client)

        # Create another instance from the volume and read the timestamp.
        timestamp_2 = self._create_or_get_timestamp(volume,
                                                    self.get_timestamp)

        self.assertEqual(timestamp_1, timestamp_2)

        # Verify the volume has a new encryption_key_id.
        encryption_key_id_1 = volume['encryption_key_id']
        volume = self.volumes_client.show_volume(volume['id'])['volume']
        encryption_key_id_2 = volume['encryption_key_id']

        self.assertNotEqual(encryption_key_id_1, encryption_key_id_2)



# Copyright (c) 2016 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from cinder_tempest_plugin.api.volume import base

CONF = config.CONF


class VolumesBackupsTest(base.BaseVolumeTest):

    @classmethod
    def skip_checks(cls):
        super(VolumesBackupsTest, cls).skip_checks()
        if not CONF.volume_feature_enabled.backup:
            raise cls.skipException("Cinder backup feature disabled")

    @classmethod
    def setup_credentials(cls):
        # Setting network=True, subnet=True creates a default network
        cls.set_network_resources(
            network=True,
            subnet=True,
            router=True,
            dhcp=True)
        super(VolumesBackupsTest, cls).setup_credentials()

    @decorators.idempotent_id('885410c6-cd1d-452c-a409-7c32b7e0be15')
    def test_volume_snapshot_backup(self):
        """Create backup from snapshot."""
        volume = self.create_volume()
        # Create snapshot
        snapshot = self.create_snapshot(volume['id'])
        # Create backup
        backup = self.create_backup(
            volume_id=volume['id'],
            snapshot_id=snapshot['id'])
        # Get a given backup. We need to do this to get the volume_id and
        # snapshot_id of the backup. They are not returned by the create API.
        backup = self.backups_client.show_backup(backup['id'])['backup']
        self.assertEqual(volume['id'], backup['volume_id'])
        self.assertEqual(snapshot['id'], backup['snapshot_id'])

    @decorators.idempotent_id('b5d837b0-7066-455d-88fc-4a721a899306')
    def test_backup_create_and_restore_to_an_existing_volume(self):
        """Test backup create and restore to an existing volume."""
        # Create volume
        src_vol = self.create_volume()
        # Create backup
        backup = self.create_backup(volume_id=src_vol['id'])
        # Restore to existing volume
        restore = self.backups_client.restore_backup(
            backup_id=backup['id'],
            volume_id=src_vol['id'])['restore']
        waiters.wait_for_volume_resource_status(
            self.backups_client,
            backup['id'], 'available')
        waiters.wait_for_volume_resource_status(
            self.volumes_client,
            src_vol['id'], 'available')
        self.assertEqual(src_vol['id'], restore['volume_id'])
        self.assertEqual(backup['id'], restore['backup_id'])

    @decorators.idempotent_id('457359e2-a663-4758-8f76-06d392dfd7c7')
    def test_incr_backup_create_and_restore_to_an_existing_volume(self):
        """Test incr backup create and restore to an existing volume."""
        # Create volume
        src_vol = self.create_volume()
        # Create a full backup
        self.create_backup(volume_id=src_vol['id'])
        # Create an inc backup
        backup = self.create_backup(volume_id=src_vol['id'],
                                    incremental=True)
        # Restore to existing volume
        restore = self.backups_client.restore_backup(
            backup_id=backup['id'],
            volume_id=src_vol['id'])['restore']
        waiters.wait_for_volume_resource_status(
            self.backups_client,
            backup['id'], 'available')
        waiters.wait_for_volume_resource_status(
            self.volumes_client,
            src_vol['id'], 'available')
        self.assertEqual(src_vol['id'], restore['volume_id'])
        self.assertEqual(backup['id'], restore['backup_id'])

    @decorators.idempotent_id('c810fe2c-cb40-43ab-96aa-471b74516a98')
    def test_incremental_backup(self):
        """Test create incremental backup."""
        # Create volume from image
        volume = self.create_volume(size=CONF.volume.volume_size,
                                    imageRef=CONF.compute.image_ref)

        # Create backup
        self.create_backup(volume_id=volume['id'])
        # Create a server
        bd_map = [{'volume_id': volume['id'],
                   'delete_on_termination': '0'}]

        server_name = data_utils.rand_name('instance')
        server = self.create_server(
            name=server_name,
            block_device_mapping=bd_map,
            wait_until='SSHABLE')

        # Delete VM
        self.os_primary.servers_client.delete_server(server['id'])
        # Create incremental backup
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')
        backup_incr = self.create_backup(
            volume_id=volume['id'],
            incremental=True)

        is_incremental = self.backups_client.show_backup(
            backup_incr['id'])['backup']['is_incremental']
        self.assertTrue(is_incremental)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from cinder_tempest_plugin.rbac.v3 import base as rbac_base
from tempest.lib import decorators
from tempest.lib import exceptions


class VolumeV3RbacCapabilityTests(rbac_base.VolumeV3RbacBaseTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.volume_capabilities_client_latest
        # NOTE(lbragstad): This admin_client will be more useful later when
        # cinder supports system-scope and we need it for administrative
        # operations. For now, keep os_project_admin as the admin client until
        # we have system-scope.
        admin_client = cls.os_project_admin
        cls.admin_capabilities_client = (
            admin_client.volume_capabilities_client_latest)
        cls.admin_stats_client = (
            admin_client.volume_scheduler_stats_client_latest)

    def _get_capabilities(self, expected_status):
        pools = self.admin_stats_client.list_pools()['pools']
        host_name = pools[0]['name']
        self.do_request(
            'show_backend_capabilities',
            expected_status=expected_status,
            host=host_name
        )


class ProjectReaderTests(VolumeV3RbacCapabilityTests):
    credentials = ['project_reader', 'project_admin', 'system_admin']

    @decorators.idempotent_id('d16034fc-4204-4ea8-94b3-714de59fdfbf')
    def test_get_capabilities(self):
        self._get_capabilities(expected_status=exceptions.Forbidden)


class ProjectMemberTests(VolumeV3RbacCapabilityTests):
    credentials = ['project_member', 'project_admin', 'system_admin']

    @decorators.idempotent_id('dbaf51de-fafa-4f55-875f-7537524489ab')
    def test_get_capabilities(self):
        self._get_capabilities(expected_status=exceptions.Forbidden)


class ProjectAdminTests(VolumeV3RbacCapabilityTests):
    credentials = ['project_admin', 'system_admin']

    @decorators.idempotent_id('1fdbe493-e58f-48bf-bb38-52003eeef8cb')
    def test_get_capabilities(self):
        self._get_capabilities(expected_status=200)



# Copyright (C) 2015 EMC Corporation.
# Copyright (C) 2016 Pure Storage, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import http.client as http_client
import time

from oslo_serialization import jsonutils as json
from tempest.lib.common import rest_client
from tempest.lib import exceptions as lib_exc

from cinder_tempest_plugin import exceptions as volume_exc


class ConsistencyGroupsClient(rest_client.RestClient):
    """Client class to send CRUD Volume ConsistencyGroup API requests"""

    def __init__(self, auth_provider, service, region, **kwargs):
        super(ConsistencyGroupsClient, self).__init__(
            auth_provider, service, region, **kwargs)

    def create_consistencygroup(self, volume_types, **kwargs):
        """Creates a consistency group."""
        post_body = {'volume_types': volume_types}
        if kwargs.get('availability_zone'):
            post_body['availability_zone'] = kwargs.get('availability_zone')
        if kwargs.get('name'):
            post_body['name'] = kwargs.get('name')
        if kwargs.get('description'):
            post_body['description'] = kwargs.get('description')
        post_body = json.dumps({'consistencygroup': post_body})
        resp, body = self.post('consistencygroups', post_body)
        body = json.loads(body)
        self.expected_success(http_client.ACCEPTED, resp.status)
        return rest_client.ResponseBody(resp, body)

    def create_consistencygroup_from_src(self, **kwargs):
        """Creates a consistency group from source."""
        post_body = {}
        if kwargs.get('cgsnapshot_id'):
            post_body['cgsnapshot_id'] = kwargs.get('cgsnapshot_id')
        if kwargs.get('source_cgid'):
            post_body['source_cgid'] = kwargs.get('source_cgid')
        if kwargs.get('name'):
            post_body['name'] = kwargs.get('name')
        if kwargs.get('description'):
            post_body['description'] = kwargs.get('description')
        post_body = json.dumps({'consistencygroup-from-src': post_body})
        resp, body = self.post('consistencygroups/create_from_src', post_body)
        body = json.loads(body)
        self.expected_success(http_client.ACCEPTED, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_consistencygroup(self, cg_id):
        """Delete a consistency group."""
        post_body = {'force': True}
        post_body = json.dumps({'consistencygroup': post_body})
        resp, body = self.post('consistencygroups/%s/delete' % cg_id,
                               post_body)
        self.expected_success(http_client.ACCEPTED, resp.status)
        return rest_client.ResponseBody(resp, body)

    def show_consistencygroup(self, cg_id):
        """Returns the details of a single consistency group."""
        url = "consistencygroups/%s" % str(cg_id)
        resp, body = self.get(url)
        body = json.loads(body)
        self.expected_success(http_client.OK, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_consistencygroups(self, detail=False):
        """Information for all the tenant's consistency groups."""
        url = "consistencygroups"
        if detail:
            url += "/detail"
        resp, body = self.get(url)
        body = json.loads(body)
        self.expected_success(http_client.OK, resp.status)
        return rest_client.ResponseBody(resp, body)

    def create_cgsnapshot(self, consistencygroup_id, **kwargs):
        """Creates a consistency group snapshot."""
        post_body = {'consistencygroup_id': consistencygroup_id}
        if kwargs.get('name'):
            post_body['name'] = kwargs.get('name')
        if kwargs.get('description'):
            post_body['description'] = kwargs.get('description')
        post_body = json.dumps({'cgsnapshot': post_body})
        resp, body = self.post('cgsnapshots', post_body)
        body = json.loads(body)
        self.expected_success(http_client.ACCEPTED, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_cgsnapshot(self, cgsnapshot_id):
        """Delete a consistency group snapshot."""
        resp, body = self.delete('cgsnapshots/%s' % (str(cgsnapshot_id)))
        self.expected_success(http_client.ACCEPTED, resp.status)
        return rest_client.ResponseBody(resp, body)

    def show_cgsnapshot(self, cgsnapshot_id):
        """Returns the details of a single consistency group snapshot."""
        url = "cgsnapshots/%s" % str(cgsnapshot_id)
        resp, body = self.get(url)
        body = json.loads(body)
        self.expected_success(http_client.OK, resp.status)
        return rest_client.ResponseBody(resp, body)

    def list_cgsnapshots(self, detail=False):
        """Information for all the tenant's consistency group snapshotss."""
        url = "cgsnapshots"
        if detail:
            url += "/detail"
        resp, body = self.get(url)
        body = json.loads(body)
        self.expected_success(http_client.OK, resp.status)
        return rest_client.ResponseBody(resp, body)

    def wait_for_consistencygroup_status(self, cg_id, status):
        """Waits for a consistency group to reach a given status."""
        body = self.show_consistencygroup(cg_id)['consistencygroup']
        cg_status = body['status']
        start = int(time.time())

        while cg_status != status:
            time.sleep(self.build_interval)
            body = self.show_consistencygroup(cg_id)['consistencygroup']
            cg_status = body['status']
            if cg_status == 'error':
                raise volume_exc.ConsistencyGroupException(cg_id=cg_id)

            if int(time.time()) - start >= self.build_timeout:
                message = ('Consistency group %s failed to reach %s status '
                           '(current %s) within the required time (%s s).' %
                           (cg_id, status, cg_status,
                            self.build_timeout))
                raise lib_exc.TimeoutException(message)

    def wait_for_consistencygroup_deletion(self, cg_id):
        """Waits for consistency group deletion"""
        start_time = int(time.time())
        while True:
            try:
                self.show_consistencygroup(cg_id)
            except lib_exc.NotFound:
                return
            if int(time.time()) - start_time >= self.build_timeout:
                raise lib_exc.TimeoutException
            time.sleep(self.build_interval)

    def wait_for_cgsnapshot_status(self, cgsnapshot_id, status):
        """Waits for a consistency group snapshot to reach a given status."""
        body = self.show_cgsnapshot(cgsnapshot_id)['cgsnapshot']
        cgsnapshot_status = body['status']
        start = int(time.time())

        while cgsnapshot_status != status:
            time.sleep(self.build_interval)
            body = self.show_cgsnapshot(cgsnapshot_id)['cgsnapshot']
            cgsnapshot_status = body['status']
            if cgsnapshot_status == 'error':
                raise volume_exc.ConsistencyGroupSnapshotException(
                    cgsnapshot_id=cgsnapshot_id)

            if int(time.time()) - start >= self.build_timeout:
                message = ('Consistency group snapshot %s failed to reach '
                           '%s status (current %s) within the required time '
                           '(%s s).' %
                           (cgsnapshot_id, status, cgsnapshot_status,
                            self.build_timeout))
                raise lib_exc.TimeoutException(message)

    def wait_for_cgsnapshot_deletion(self, cgsnapshot_id):
        """Waits for consistency group snapshot deletion"""
        start_time = int(time.time())
        while True:
            try:
                self.show_cgsnapshot(cgsnapshot_id)
            except lib_exc.NotFound:
                return
            if int(time.time()) - start_time >= self.build_timeout:
                raise lib_exc.TimeoutException
            time.sleep(self.build_interval)




#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import waiters
from tempest import config
from tempest.lib.common import api_version_utils
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib.decorators import cleanup_order
from tempest import test

CONF = config.CONF


class VolumeV3RbacBaseTests(
    api_version_utils.BaseMicroversionTest, test.BaseTestCase
):
    identity_version = 'v3'

    @classmethod
    def skip_checks(cls):
        super(VolumeV3RbacBaseTests, cls).skip_checks()
        if not CONF.enforce_scope.cinder:
            raise cls.skipException(
                "Tempest is not configured to enforce_scope for cinder, "
                "skipping RBAC tests. To enable these tests set "
                "`tempest.conf [enforce_scope] cinder=True`."
            )
        if not CONF.service_available.cinder:
            skip_msg = ("%s skipped as Cinder is not available" % cls.__name__)
            raise cls.skipException(skip_msg)
        api_version_utils.check_skip_with_microversion(
            cls.min_microversion, cls.max_microversion,
            CONF.volume.min_microversion, CONF.volume.max_microversion)

    @classmethod
    def setup_credentials(cls):
        cls.set_network_resources()
        super(VolumeV3RbacBaseTests, cls).setup_credentials()

    def setUp(self):
        super(VolumeV3RbacBaseTests, self).setUp()

    @classmethod
    def resource_setup(cls):
        super(VolumeV3RbacBaseTests, cls).resource_setup()
        cls.request_microversion = (
            api_version_utils.select_request_microversion(
                cls.min_microversion,
                CONF.volume.min_microversion))
        cls.setup_api_microversion_fixture(
            volume_microversion=cls.request_microversion)

    def do_request(self, method, expected_status=200, client=None, **payload):
        """Perform API call

        Args:
            method: Name of the API call
            expected_status: HTTP desired response code
            client: Client object if exists, None otherwise
            payload: API call required parameters

        Returns:
            HTTP response
        """
        if not client:
            client = self.client
        if isinstance(expected_status, type(Exception)):
            self.assertRaises(expected_status,
                              getattr(client, method),
                              **payload)
        else:
            response = getattr(client, method)(**payload)
            self.assertEqual(response.response.status, expected_status)
            return response

    @cleanup_order
    def create_volume(self, client, **kwargs):
        """Wrapper utility that returns a test volume

        Args:
            client: Client object

        Returns:
            ID of the created volume
        """
        kwargs['size'] = CONF.volume.volume_size
        kwargs['name'] = data_utils.rand_name(
            VolumeV3RbacBaseTests.__name__ + '-Volume'
        )

        volume_id = client.create_volume(**kwargs)['volume']['id']
        self.cleanup(
            test_utils.call_and_ignore_notfound_exc, func=self.delete_resource,
            client=client, volume_id=volume_id
        )
        waiters.wait_for_volume_resource_status(
            client=client, resource_id=volume_id, status='available'
        )

        return volume_id

    @cleanup_order
    def create_snapshot(self, client, volume_id, cleanup=True, **kwargs):
        """Wrapper utility that returns a test snapshot.

        Args:
            client: Client object
            volume_id: ID of the volume
            cleanup: Boolean if should delete the snapshot

        Returns:
            ID of the created snapshot
        """
        kwargs['name'] = data_utils.rand_name(
            VolumeV3RbacBaseTests.__name__ + '-Snapshot'
        )

        snapshot_id = client.create_snapshot(
            volume_id=volume_id, **kwargs)['snapshot']['id']
        if cleanup:
            self.cleanup(
                test_utils.call_and_ignore_notfound_exc,
                func=self.delete_resource,
                client=client, snapshot_id=snapshot_id
            )
        waiters.wait_for_volume_resource_status(
            client=client, resource_id=snapshot_id, status='available'
        )

        return snapshot_id

    @classmethod
    def delete_resource(cls, client, **kwargs):
        """Delete a resource by a given client

        Args:
            client: Client object

        Keyword Args:
            snapshot_id: ID of a snapshot
            volume_id: ID of a volume
        """
        key, resource_id = list(kwargs.items())[0]
        resource_name = key.split('_')[0]

        del_action = getattr(client, f'delete_{resource_name}')
        test_utils.call_and_ignore_notfound_exc(del_action, resource_id)
        test_utils.call_and_ignore_notfound_exc(
            client.wait_for_resource_deletion, resource_id)

    @classmethod
    def create_backup(
            cls, volume_id, backup_client=None, add_cleanup=True, **kwargs
    ):
        """Wrapper utility that returns a test backup."""
        if backup_client is None:
            backup_client = cls.backups_client
        if 'name' not in kwargs:
            name = data_utils.rand_name(cls.__class__.__name__ + '-Backup')
            kwargs['name'] = name

        backup = backup_client.create_backup(
            volume_id=volume_id, **kwargs
        )['backup']
        if add_cleanup:
            cls.addClassResourceCleanup(
                test_utils.call_and_ignore_notfound_exc,
                cls.delete_resource,
                client=backup_client,
                backup_id=backup['id']
            )
        waiters.wait_for_volume_resource_status(
            backup_client, backup['id'], 'available'
        )
        return backup



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions

from cinder_tempest_plugin.rbac.v3 import base as rbac_base

CONF = config.CONF


class VolumeV3RbacVolumeActionsTests(rbac_base.VolumeV3RbacBaseTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.vol_other_client = cls.os_project_admin.volumes_client_latest

    def _extend_volume(self, expected_status):
        """Test extend_volume operation.

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        self.do_request(
            method='extend_volume', volume_id=volume_id,
            new_size=2, expected_status=expected_status
        )

    def _reset_volume_status(self, expected_status):
        """Test reset_volume_status operation.

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        self.do_request(
            method='reset_volume_status', volume_id=volume_id,
            status='error', expected_status=expected_status
        )

    def _retype_volume(self, expected_status):
        """Test retype_volume operation.

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        self.do_request(
            method='retype_volume', volume_id=volume_id,
            new_type='dedup-tier-replication', expected_status=expected_status
        )

    def _update_volume_readonly(self, expected_status):
        """Test update_volume_readonly operation.

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        self.do_request(
            method='update_volume_readonly', volume_id=volume_id,
            readonly=True, expected_status=expected_status
        )

    def _force_delete_volume(self, expected_status):
        """Test force_delete_volume operation.

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        self.do_request(
            method='force_delete_volume', volume_id=volume_id,
            expected_status=expected_status
        )

    def _reserve_volume(self, expected_status):
        """Test reserve_volume operation.

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        self.do_request(
            method='reserve_volume', volume_id=volume_id,
            expected_status=expected_status
        )

    def _unreserve_volume(self, expected_status):
        """Test unreserve_volume operation.

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        self.do_request(
            method='unreserve_volume', volume_id=volume_id,
            expected_status=expected_status
        )


class ProjectReaderTests(VolumeV3RbacVolumeActionsTests):

    credentials = ['project_reader', 'project_admin']

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_reader.volumes_client_latest

    @decorators.skip_because(bug="2020261")
    @decorators.idempotent_id('4d721c58-2f6f-4857-8f4f-0664d5f7bf49')
    def test_extend_volume(self):
        self._extend_volume(expected_status=exceptions.Forbidden)

    @decorators.idempotent_id('434b454a-5cbe-492d-a416-70b8ff41f636')
    def test_reset_volume_status(self):
        self._reset_volume_status(expected_status=exceptions.Forbidden)

    @decorators.skip_because(bug="2020261")
    @decorators.idempotent_id('4675295a-7c72-4b04-8a43-03d7c88ab6bf')
    def test_retype_volume(self):
        self._retype_volume(expected_status=exceptions.Forbidden)

    @decorators.skip_because(bug="2020261")
    @decorators.idempotent_id('3beecd52-e314-40d8-875d-a0e7db8dd88f')
    def test_update_volume_readonly(self):
        self._update_volume_readonly(expected_status=exceptions.Forbidden)

    @decorators.idempotent_id('b025ff12-73a4-4f15-af55-876cd43cade3')
    def test_force_delete_volume(self):
        self._force_delete_volume(expected_status=exceptions.Forbidden)

    @decorators.skip_because(bug="2020261")
    @decorators.idempotent_id('d2c13bf9-267a-4a71-be5c-391f22e9b433')
    def test_reserve_volume(self):
        self._reserve_volume(expected_status=exceptions.Forbidden)

    @decorators.skip_because(bug="2020261")
    @decorators.idempotent_id('725d85cf-96b2-4338-98f4-2f468099c4ed')
    def test_unreserve_volume(self):
        self._unreserve_volume(expected_status=exceptions.Forbidden)



# Copyright 2022 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import waiters
from tempest import config
from tempest.lib import decorators

from cinder_tempest_plugin.api.volume import base

CONF = config.CONF


class CreateVolumesFromSnapshotTest(base.CreateMultipleResourceTest):

    @decorators.idempotent_id('3b879ad1-d861-4ad3-b2c8-c89162e867c3')
    def test_create_multiple_volume_from_snapshot(self):
        """Create multiple volumes from a snapshot."""

        volume = self.create_volume()
        snapshot = self.create_snapshot(volume_id=volume['id'])
        kwargs_create = {"'snapshot_id": snapshot['id'], "wait_until": None}
        res = self._create_multiple_resource(self.create_volume,
                                             **kwargs_create)
        kwargs_wait = {"client": self.volumes_client, "status": "available"}
        self._wait_for_multiple_resources(
            waiters.wait_for_volume_resource_status, res, **kwargs_wait)


class CreateVolumesFromSourceVolumeTest(base.CreateMultipleResourceTest):

    @decorators.idempotent_id('b4a250d1-3ffd-4727-a2f5-9d858b298558')
    def test_create_multiple_volume_from_source_volume(self):
        """Create multiple volumes from a source volume.

        The purpose of this test is to check the synchronization
        of driver clone method with simultaneous requests.
        """

        volume = self.create_volume()
        kwargs_create = {"'source_volid": volume['id'], "wait_until": None}
        res = self._create_multiple_resource(self.create_volume,
                                             **kwargs_create)
        kwargs_wait = {"client": self.volumes_client, "status": "available"}
        self._wait_for_multiple_resources(
            waiters.wait_for_volume_resource_status, res, **kwargs_wait)


class CreateVolumesFromBackupTest(base.CreateMultipleResourceTest):

    @classmethod
    def skip_checks(cls):
        super(CreateVolumesFromBackupTest, cls).skip_checks()
        if not CONF.volume_feature_enabled.backup:
            raise cls.skipException("Cinder backup feature disabled")

    @decorators.idempotent_id('9db67083-bf1a-486c-8f77-3778467f39a1')
    def test_create_multiple_volume_from_backup(self):
        """Create multiple volumes from a backup."""

        volume = self.create_volume()
        backup = self.create_backup(volume_id=volume['id'])
        kwargs_create = {"'backup_id": backup['id'], "wait_until": None}
        res = self._create_multiple_resource(self.create_volume,
                                             **kwargs_create)
        kwargs_wait = {"client": self.volumes_client, "status": "available"}
        self._wait_for_multiple_resources(
            waiters.wait_for_volume_resource_status, res, **kwargs_wait)


class CreateVolumesFromImageTest(base.CreateMultipleResourceTest):

    @classmethod
    def skip_checks(cls):
        super(CreateVolumesFromImageTest, cls).skip_checks()
        if not CONF.service_available.glance:
            raise cls.skipException("Glance service is disabled")

    @decorators.idempotent_id('8976a11b-1ddc-49b6-b66f-8c26adf3fa9e')
    def test_create_from_image_multiple(self):
        """Create a handful of volumes from the same image at once.

        The purpose of this test is to stress volume drivers,
        image download, the image cache, etc., within Cinder.
        """

        img_uuid = CONF.compute.image_ref

        kwargs_create = {"'imageRef": img_uuid, "wait_until": None}
        res = self._create_multiple_resource(self.create_volume,
                                             **kwargs_create)
        kwargs_wait = {"client": self.volumes_client, "status": "available"}
        self._wait_for_multiple_resources(
            waiters.wait_for_volume_resource_status, res, **kwargs_wait)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions

import testtools

from cinder_tempest_plugin.rbac.v3 import base as rbac_base

CONF = config.CONF


class VolumeV3RbacSnapshotsTests(rbac_base.VolumeV3RbacBaseTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.vol_other_client = cls.os_project_admin.volumes_client_latest
        cls.snap_other_client = cls.os_project_admin.snapshots_client_latest

    def _list_snapshots(self, expected_status):
        """Test list_snapshots operation

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        self.create_snapshot(
            client=self.snap_other_client, volume_id=volume_id
        )
        self.do_request(
            expected_status=expected_status, method='list_snapshots'
        )

    def _show_snapshot(self, expected_status):
        """Test show_snapshot operation

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        snapshot_id = self.create_snapshot(
            client=self.snap_other_client, volume_id=volume_id
        )
        self.do_request(
            expected_status=expected_status, method='show_snapshot',
            snapshot_id=snapshot_id
        )

    def _create_snapshot(self, expected_status):
        """Test create_snapshot operation.

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        snap_name = data_utils.rand_name(
            self.__name__ + '-Snapshot'
        )
        if expected_status == 202:
            snapshot_id = self.do_request(
                method='create_snapshot', expected_status=202,
                volume_id=volume_id, name=snap_name
            )['snapshot']['id']
            self.addCleanup(
                test_utils.call_and_ignore_notfound_exc, self.delete_resource,
                client=self.client, snapshot_id=snapshot_id
            )
            waiters.wait_for_volume_resource_status(
                client=self.client, resource_id=snapshot_id, status='available'
            )
        elif expected_status == exceptions.Forbidden:
            self.do_request(
                method='create_snapshot', expected_status=expected_status,
                volume_id=volume_id, name=snap_name
            )

    def _remove_snapshot(self, expected_status):
        """Test create_snapshot operation.

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        snapshot_id = self.create_snapshot(
            client=self.snap_other_client, volume_id=volume_id
        )

        self.do_request(
            method='delete_snapshot', snapshot_id=snapshot_id,
            expected_status=expected_status
        )
        if expected_status == 202:
            self.client.wait_for_resource_deletion(id=snapshot_id)

    def _reset_snapshot_status(self, expected_status):
        """Test reset_snapshot_status operation.

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        snapshot_id = self.create_snapshot(
            client=self.snap_other_client, volume_id=volume_id
        )
        self.do_request(
            'reset_snapshot_status', expected_status=expected_status,
            snapshot_id=snapshot_id, status='error'
        )

    def _update_snapshot(self, expected_status):
        """Test update_snapshot operation.

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        snapshot_id = self.create_snapshot(
            client=self.snap_other_client, volume_id=volume_id
        )
        new_desc = self.__name__ + '-update_test'
        self.do_request(
            method='update_snapshot', expected_status=expected_status,
            snapshot_id=snapshot_id, description=new_desc
        )

    def _update_snapshot_status(self, expected_status):
        """Test update_snapshot_status operation.

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        snapshot_id = self.create_snapshot(
            client=self.snap_other_client, volume_id=volume_id
        )

        reset_status = 'creating' if expected_status == 202 else 'error'
        request_status = 'error' if expected_status == 202 else 'creating'
        self.os_project_admin.snapshots_client_latest.reset_snapshot_status(
            snapshot_id=snapshot_id, status=reset_status
        )
        waiters.wait_for_volume_resource_status(
            client=self.os_project_admin.snapshots_client_latest,
            resource_id=snapshot_id, status=reset_status
        )

        self.do_request(
            'update_snapshot_status', expected_status=expected_status,
            snapshot_id=snapshot_id, status=request_status, progress='80%'
        )

    def _force_delete_snapshot(self, expected_status):
        """Test force_delete_snapshot operation.

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        snapshot_id = self.create_snapshot(
            client=self.snap_other_client, volume_id=volume_id
        )
        self.do_request(
            method='force_delete_snapshot', snapshot_id=snapshot_id,
            expected_status=expected_status
        )
        if expected_status != exceptions.Forbidden:
            self.client.wait_for_resource_deletion(id=snapshot_id)
            waiters.wait_for_volume_resource_status(
                client=self.os_project_admin.volumes_client_latest,
                resource_id=volume_id, status='available'
            )

    def _unmanage_snapshot(self, expected_status):
        """Test unmanage_snapshot operation.

        Args:
            expected_status: The expected HTTP response code
        """
        volume_id = self.create_volume(client=self.vol_other_client)
        snapshot_id = self.create_snapshot(
            client=self.snap_other_client, volume_id=volume_id
        )
        self.do_request(
            method='unmanage_snapshot',
            expected_status=expected_status, snapshot_id=snapshot_id
        )
        if expected_status != exceptions.Forbidden:
            self.client.wait_for_resource_deletion(id=snapshot_id)

    def _manage_snapshot(self, client, expected_status):
        """Test reset_snapshot_status operation.

        Args:
            client: The client to perform the needed request
            expected_status: The expected HTTP response code
        """
        # Create a volume
        volume_id = self.create_volume(client=self.vol_other_client)

        # Create a snapshot
        snapshot_id = self.create_snapshot(
            client=self.snap_other_client,
            volume_id=volume_id,
            cleanup=False
        )
        # Unmanage the snapshot
        # Unmanage snapshot function works almost the same as delete snapshot,
        # but it does not delete the snapshot data
        self.snap_other_client.unmanage_snapshot(snapshot_id)
        self.client.wait_for_resource_deletion(snapshot_id)

        # Verify the original snapshot does not exist in snapshot list
        params = {'all_tenants': 1}
        all_snapshots = self.snap_other_client.list_snapshots(
            detail=True, **params)['snapshots']
        self.assertNotIn(snapshot_id, [v['id'] for v in all_snapshots])

        # Manage the snapshot
        name = data_utils.rand_name(
            self.__class__.__name__ + '-Managed-Snapshot'
        )
        description = data_utils.rand_name(
            self.__class__.__name__ + '-Managed-Snapshot-Description'
        )
        metadata = {"manage-snap-meta1": "value1",
                    "manage-snap-meta2": "value2",
                    "manage-snap-meta3": "value3"}
        snapshot_ref = {
            'volume_id': volume_id,
            'ref': {CONF.volume.manage_snapshot_ref[0]:
                    CONF.volume.manage_snapshot_ref[1] % snapshot_id},
            'name': name,
            'description': description,
            'metadata': metadata
        }

        new_snapshot = self.do_request(
            client=client,
            method='manage_snapshot', expected_status=expected_status,
            volume_id=volume_id, ref=snapshot_ref
        )
        if expected_status != exceptions.Forbidden:
            snapshot = new_snapshot['snapshot']
            waiters.wait_for_volume_resource_status(
                client=self.snap_other_client,
                resource_id=snapshot['id'],
                status='available'
            )
            self.delete_resource(
                client=self.snap_other_client, snapshot_id=snapshot['id']
            )


class ProjectReaderTests(VolumeV3RbacSnapshotsTests):

    credentials = ['project_reader', 'project_admin']

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_reader.snapshots_client_latest

    @decorators.idempotent_id('dd8e19dc-c8fd-443c-8aed-cdffe07fa6be')
    def test_list_snapshots(self):
        self._list_snapshots(expected_status=200)

    @decorators.idempotent_id('6f69e8ed-4e11-40a1-9620-258cf3c45872')
    def test_show_snapshot(self):
        self._show_snapshot(expected_status=200)

    @decorators.skip_because(bug="2017108")
    @decorators.idempotent_id('13ae344f-fa01-44cc-b9f1-d04452940dc1')
    def test_create_snapshot(self):
        self._create_snapshot(expected_status=exceptions.Forbidden)

    @decorators.skip_because(bug="2017108")
    @decorators.idempotent_id('5b58f647-da0f-4d2a-bf68-680fc692efb4')
    def test_delete_snapshot(self):
        self._remove_snapshot(expected_status=exceptions.Forbidden)

    @decorators.idempotent_id('809d8c8c-25bf-4f1f-9b77-1a81ce4292d1')
    def test_reset_snapshot_status(self):
        self._reset_snapshot_status(expected_status=exceptions.Forbidden)

    @decorators.skip_because(bug="2017108")
    @decorators.idempotent_id('c46f5df8-9a6f-4ed6-b94c-3b65ef05ee9e')
    def test_update_snapshot(self):
        self._update_snapshot(expected_status=exceptions.Forbidden)

    @decorators.skip_because(bug="2017108")
    @decorators.idempotent_id('c90f98d7-3665-4c9f-820f-3f4c2adfdbf5')
    def test_update_snapshot_status(self):
        self._update_snapshot_status(expected_status=exceptions.Forbidden)

    @decorators.idempotent_id('63aa8184-897d-4e00-9b80-d2e7828f1b13')
    def test_force_delete_snapshot(self):
        self._force_delete_snapshot(expected_status=exceptions.Forbidden)

    # We don't need to skip the unmanage because the failure should happen at
    # the API level, so it should fail regardless of the driver support for
    # unmanaging snapshots.
    @decorators.idempotent_id('35495666-b663-4c68-ba44-0695e30a6838')
    def test_unmanage_snapshot(self):
        self._unmanage_snapshot(expected_status=exceptions.Forbidden)

    @testtools.skipUnless(CONF.volume_feature_enabled.manage_snapshot,
                          'manage snapshots are disabled')
    @decorators.idempotent_id('d2d1326d-fb47-4448-a1e1-2d1219d30fd5')
    def test_manage_snapshot(self):
        self._manage_snapshot(
            expected_status=exceptions.Forbidden,
            client=self.os_project_reader.snapshot_manage_client_latest
        )


class ProjectMemberTests(VolumeV3RbacSnapshotsTests):

    credentials = ['project_member', 'project_admin']

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_member.snapshots_client_latest

    @decorators.idempotent_id('5b3ec87f-443f-42f7-bd3c-ab05ea30c5e1')
    def test_list_snapshots(self):
        self._list_snapshots(expected_status=200)

    @decorators.idempotent_id('6fee8967-951c-4957-b51b-97b83c13c7c3')
    def test_show_snapshot(self):
        self._show_snapshot(expected_status=200)

    @decorators.idempotent_id('43f77b31-aab4-46d0-b76f-e17000d23589')
    def test_create_snapshot(self):
        self._create_snapshot(expected_status=202)

    @decorators.idempotent_id('22939122-8b4e-47d5-abaa-774bc55c07fc')
    def test_delete_snapshot(self):
        self._remove_snapshot(expected_status=202)

    @decorators.idempotent_id('da391afd-8baa-458b-b222-f6ab42ab47c3')
    def test_reset_snapshot_status(self):
        self._reset_snapshot_status(expected_status=exceptions.Forbidden)

    @decorators.idempotent_id('a774bdca-bfbe-477d-9711-5fb64d7e34ea')
    def test_update_snapshot(self):
        self._update_snapshot(expected_status=200)

    @decorators.idempotent_id('12e00e1b-bf84-41c1-8a1e-8625d1317789')
    def test_update_snapshot_status(self):
        self._update_snapshot_status(expected_status=202)

    @decorators.idempotent_id('e7cb3eb0-d607-4c90-995d-df82d030eca8')
    def test_force_delete_snapshot(self):
        self._force_delete_snapshot(expected_status=exceptions.Forbidden)

    # We don't need to skip the unmanage because the failure should happen at
    # the API level, so it should fail regardless of the driver support for
    # unmanaging snapshots.
    @decorators.idempotent_id('dd7da3da-68ef-42f5-af1d-29803a4a04fd')
    def test_unmanage_snapshot(self):
        self._unmanage_snapshot(expected_status=exceptions.Forbidden)

    @testtools.skipUnless(CONF.volume_feature_enabled.manage_snapshot,
                          'manage snapshots are disabled')
    @decorators.idempotent_id('c2501d05-9bca-42d7-9ab5-c0d9133e762f')
    def test_manage_snapshot(self):
        self._manage_snapshot(
            expected_status=exceptions.Forbidden,
            client=self.os_project_member.snapshot_manage_client_latest
        )



# Copyright 2022 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators
import testtools

from cinder_tempest_plugin.api.volume import base

CONF = config.CONF


class VolumeDependencyTests(base.BaseVolumeTest):
    min_microversion = '3.40'

    @classmethod
    def setup_clients(cls):
        super(VolumeDependencyTests, cls).setup_clients()

    @decorators.idempotent_id('42e9df95-854b-4840-9d55-ae62f65e9b8e')
    def test_delete_source_volume(self):
        """Test basic dependency deletion

        * Create a volume with source_volid
        * Delete the source volume
        """
        source_volume = self.create_volume()
        kwargs = {'source_volid': source_volume['id']}
        cloned_volume = self.create_volume(**kwargs)
        self.assertEqual(source_volume['id'], cloned_volume['source_volid'])
        self.volumes_client.delete_volume(source_volume['id'])
        self.volumes_client.wait_for_resource_deletion(source_volume['id'])

    @decorators.idempotent_id('900d8ea5-2afd-4fe5-a0c3-fab4744f0d40')
    def test_delete_source_snapshot(self):
        """Test basic dependency deletion with snapshot

        * Create a snapshot from source volume
        * Create a volume from that snapshot
        * Delete the source snapshot
        * Delete the source volume
        """
        source_volume = self.create_volume()
        snapshot_source_volume = self.create_snapshot(source_volume['id'])
        kwargs = {'snapshot_id': snapshot_source_volume['id']}
        volume_from_snapshot = self.create_volume(**kwargs)
        self.assertEqual(volume_from_snapshot['snapshot_id'],
                         snapshot_source_volume['id'])

        self.snapshots_client.delete_snapshot(snapshot_source_volume['id'])
        self.snapshots_client.wait_for_resource_deletion(
            snapshot_source_volume['id'])
        self.volumes_client.delete_volume(source_volume['id'])
        self.volumes_client.wait_for_resource_deletion(source_volume['id'])

    def _delete_vol_and_wait(self, vol_id):
        self.volumes_client.delete_volume(vol_id)

        self.volumes_client.wait_for_resource_deletion(vol_id)

    def _delete_snap_and_wait(self, snap_id):
        self.snapshots_client.delete_snapshot(snap_id)

        self.snapshots_client.wait_for_resource_deletion(snap_id)

    @decorators.idempotent_id('f8278e5c-50ff-4a1d-8670-3ca0866d411a')
    def test_delete_dep_chain(self):
        """Test a complex chain of volume and snapshot dependency deletion."""
        volume_1 = self.create_volume()['id']
        snapshot_of_vol_1 = self.create_snapshot(volume_1)['id']

        volume_2_args = {'snapshot_id': snapshot_of_vol_1}
        volume_2 = self.create_volume(**volume_2_args)['id']

        snapshot_of_vol_2 = self.create_snapshot(volume_2)['id']

        volume_3_args = {'snapshot_id': snapshot_of_vol_2}
        volume_3 = self.create_volume(**volume_3_args)['id']

        volume_4_args = {'source_volid': volume_3}
        volume_4 = self.create_volume(**volume_4_args)['id']

        self._delete_snap_and_wait(snapshot_of_vol_1)
        self._delete_snap_and_wait(snapshot_of_vol_2)

        self._delete_vol_and_wait(volume_3)
        self._delete_vol_and_wait(volume_1)
        self._delete_vol_and_wait(volume_2)
        self._delete_vol_and_wait(volume_4)

    @decorators.idempotent_id('63447ef8-e667-4796-ba66-1b9b883af1f1')
    def test_delete_dep_chain_2(self):
        """Test a different chain of volume/snapshot dependency deletion."""
        volume_1 = self.create_volume()['id']
        snapshot_of_vol_1 = self.create_snapshot(volume_1)['id']

        volume_2_args = {'snapshot_id': snapshot_of_vol_1}
        volume_2 = self.create_volume(**volume_2_args)['id']

        snapshot_of_vol_2 = self.create_snapshot(volume_2)['id']

        volume_3_args = {'snapshot_id': snapshot_of_vol_2}
        volume_3 = self.create_volume(**volume_3_args)['id']

        self._delete_snap_and_wait(snapshot_of_vol_1)
        self._delete_snap_and_wait(snapshot_of_vol_2)

        self._delete_vol_and_wait(volume_1)
        self._delete_vol_and_wait(volume_2)
        self._delete_vol_and_wait(volume_3)


class VolumeImageDependencyTests(base.BaseVolumeTest):
    """Volume<->image dependency tests.

    These tests perform clones to/from volumes and images,
    deleting images/volumes that other volumes were cloned from.

    Images and volumes are expected to be independent at the OpenStack
    level, but in some configurations (i.e. when using Ceph as storage
    for both Cinder and Glance) it was possible to end up with images
    or volumes that could not be deleted.  This was fixed for RBD in
    Cinder 2024.1 change I009d0748f.

    """

    min_microversion = '3.40'

    @classmethod
    def del_image(cls, image_id):
        images_client = cls.os_primary.image_client_v2
        images_client.delete_image(image_id)
        images_client.wait_for_resource_deletion(image_id)

    @testtools.skipUnless(
        CONF.volume_feature_enabled.enable_volume_image_dep_tests,
        reason='Volume/image dependency tests not enabled.')
    @utils.services('image', 'volume')
    @decorators.idempotent_id('7a9fba78-2e4b-42b1-9898-bb4a60685320')
    def test_image_volume_dependencies_1(self):
        # image -> volume
        image_args = {
            'disk_format': 'raw',
            'container_format': 'bare',
            'name': 'image-for-test-7a9fba78-2e4b-42b1-9898-bb4a60685320'
        }
        image = self.create_image_with_data(**image_args)

        # create a volume from the image
        vol_args = {'name': ('volume1-for-test'
                             '7a9fba78-2e4b-42b1-9898-bb4a60685320'),
                    'imageRef': image['id']}
        volume1 = self.create_volume(**vol_args)
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume1['id'],
                                                'available')

        self.volumes_client.delete_volume(volume1['id'])
        self.volumes_client.wait_for_resource_deletion(volume1['id'])

        self.del_image(image['id'])

    @testtools.skipUnless(
        CONF.volume_feature_enabled.enable_volume_image_dep_tests,
        reason='Volume/image dependency tests not enabled.')
    @utils.services('image', 'volume')
    @decorators.idempotent_id('0e20bd6e-440f-41d8-9b5d-fc047ac00423')
    def test_image_volume_dependencies_2(self):
        # image -> volume -> volume

        image_args = {
            'disk_format': 'raw',
            'container_format': 'bare',
            'name': 'image-for-test-0e20bd6e-440f-41d8-9b5d-fc047ac00423'
        }
        image = self.create_image_with_data(**image_args)

        # create a volume from the image
        vol_args = {'name': ('volume1-for-test'
                             '0e20bd6e-440f-41d8-9b5d-fc047ac00423'),
                    'imageRef': image['id']}
        volume1 = self.create_volume(**vol_args)
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume1['id'],
                                                'available')

        vol2_args = {'name': ('volume2-for-test-'
                              '0e20bd6e-440f-41d8-9b5d-fc047ac00423'),
                     'source_volid': volume1['id']}
        volume2 = self.create_volume(**vol2_args)
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume2['id'],
                                                'available')

        self.volumes_client.delete_volume(volume1['id'])
        self.volumes_client.wait_for_resource_deletion(volume1['id'])

        self.del_image(image['id'])

    @testtools.skipUnless(
        CONF.volume_feature_enabled.enable_volume_image_dep_tests,
        reason='Volume/image dependency tests not enabled.')
    @decorators.idempotent_id('e6050452-06bd-4c7f-9912-45178c83e379')
    @utils.services('image', 'volume')
    def test_image_volume_dependencies_3(self):
        # image -> volume -> snap -> volume

        image_args = {
            'disk_format': 'raw',
            'container_format': 'bare',
            'name': 'image-for-test-e6050452-06bd-4c7f-9912-45178c83e379'
        }
        image = self.create_image_with_data(**image_args)

        # create a volume from the image
        vol_args = {'name': ('volume1-for-test'
                             'e6050452-06bd-4c7f-9912-45178c83e379'),
                    'imageRef': image['id']}
        volume1 = self.create_volume(**vol_args)
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume1['id'],
                                                'available')

        snapshot1 = self.create_snapshot(volume1['id'])

        vol2_args = {'name': ('volume2-for-test-'
                              'e6050452-06bd-4c7f-9912-45178c83e379'),
                     'snapshot_id': snapshot1['id']}
        volume2 = self.create_volume(**vol2_args)
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume2['id'],
                                                'available')

        self.snapshots_client.delete_snapshot(snapshot1['id'])
        self.snapshots_client.wait_for_resource_deletion(snapshot1['id'])

        self.volumes_client.delete_volume(volume2['id'])
        self.volumes_client.wait_for_resource_deletion(volume2['id'])

        self.del_image(image['id'])

        self.volumes_client.delete_volume(volume1['id'])
        self.volumes_client.wait_for_resource_deletion(volume1['id'])



# Copyright (C) 2015 EMC Corporation.
# Copyright (C) 2016 Pure Storage, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from cinder_tempest_plugin.api.volume import base

CONF = config.CONF


class ConsistencyGroupsV2Test(base.BaseVolumeAdminTest):

    @classmethod
    def skip_checks(cls):
        super(ConsistencyGroupsV2Test, cls).skip_checks()
        if not CONF.volume_feature_enabled.consistency_group:
            raise cls.skipException("Cinder consistency group "
                                    "feature disabled")

    def _delete_consistencygroup(self, cg_id):
        self.admin_consistencygroups_client.delete_consistencygroup(cg_id)
        vols = self.admin_volume_client.list_volumes(detail=True)['volumes']
        for vol in vols:
            if vol['consistencygroup_id'] == cg_id:
                self.admin_volume_client.wait_for_resource_deletion(vol['id'])
        self.admin_consistencygroups_client.wait_for_consistencygroup_deletion(
            cg_id)

    def _delete_cgsnapshot(self, cgsnapshot_id, cg_id):
        self.admin_consistencygroups_client.delete_cgsnapshot(cgsnapshot_id)
        vols = self.admin_volume_client.list_volumes(detail=True)['volumes']
        snapshots = self.os_admin.snapshots_v2_client.list_snapshots(
            detail=True)['snapshots']
        for vol in vols:
            for snap in snapshots:
                if (vol['consistencygroup_id'] == cg_id and
                        vol['id'] == snap['volume_id']):
                    (self.snapshots_client.
                     wait_for_resource_deletion(snap['id']))
        self.admin_consistencygroups_client.wait_for_cgsnapshot_deletion(
            cgsnapshot_id)

    @decorators.idempotent_id('3fe776ba-ec1f-4e6c-8d78-4b14c3a7fc44')
    def test_consistencygroup_create_delete(self):
        # Create volume type
        name = data_utils.rand_name("volume-type")
        volume_type = self.os_admin.volume_types_v2_client.create_volume_type(
            name=name)['volume_type']

        # Create CG
        cg_name = data_utils.rand_name('CG')
        create_consistencygroup = (
            self.admin_consistencygroups_client.create_consistencygroup)
        cg = create_consistencygroup(volume_type['id'],
                                     name=cg_name)['consistencygroup']
        self.admin_consistencygroups_client.wait_for_consistencygroup_status(
            cg['id'], 'available')
        self.assertEqual(cg_name, cg['name'])

        # Create volume
        vol_name = data_utils.rand_name("volume")
        params = {'name': vol_name,
                  'volume_type': volume_type['id'],
                  'consistencygroup_id': cg['id'],
                  'size': CONF.volume.volume_size}

        volume = self.admin_volume_client.create_volume(**params)['volume']
        waiters.wait_for_volume_resource_status(self.admin_volume_client,
                                                volume['id'], 'available')

        # Get a given CG
        cg = self.admin_consistencygroups_client.show_consistencygroup(
            cg['id'])['consistencygroup']
        self.assertEqual(cg_name, cg['name'])

        # Get all CGs with detail
        cgs = self.admin_consistencygroups_client.list_consistencygroups(
            detail=True)['consistencygroups']
        self.assertIn((cg['name'], cg['id']),
                      [(m['name'], m['id']) for m in cgs])

        # Clean up
        self._delete_consistencygroup(cg['id'])
        self.os_admin.volume_types_v2_client.delete_volume_type(
            volume_type['id'])

    @decorators.idempotent_id('2134dd52-f333-4456-bb05-6cb0f009a44f')
    def test_consistencygroup_cgsnapshot_create_delete(self):
        # Create volume type
        name = data_utils.rand_name("volume-type")
        volume_type = self.admin_volume_types_client.create_volume_type(
            name=name)['volume_type']

        # Create CG
        cg_name = data_utils.rand_name('CG')
        create_consistencygroup = (
            self.admin_consistencygroups_client.create_consistencygroup)
        cg = create_consistencygroup(volume_type['id'],
                                     name=cg_name)['consistencygroup']
        self.admin_consistencygroups_client.wait_for_consistencygroup_status(
            cg['id'], 'available')
        self.assertEqual(cg_name, cg['name'])

        # Create volume
        vol_name = data_utils.rand_name("volume")
        params = {'name': vol_name,
                  'volume_type': volume_type['id'],
                  'consistencygroup_id': cg['id'],
                  'size': CONF.volume.volume_size}
        volume = self.admin_volume_client.create_volume(**params)['volume']
        waiters.wait_for_volume_resource_status(self.admin_volume_client,
                                                volume['id'], 'available')

        # Create cgsnapshot
        cgsnapshot_name = data_utils.rand_name('cgsnapshot')
        create_cgsnapshot = (
            self.admin_consistencygroups_client.create_cgsnapshot)
        cgsnapshot = create_cgsnapshot(cg['id'],
                                       name=cgsnapshot_name)['cgsnapshot']
        self.admin_consistencygroups_client.wait_for_cgsnapshot_status(
            cgsnapshot['id'], 'available')
        self.assertEqual(cgsnapshot_name, cgsnapshot['name'])
        snapshots = self.os_admin.snapshots_v2_client.list_snapshots(
            detail=True)['snapshots']
        for snap in snapshots:
            if volume['id'] == snap['volume_id']:
                waiters.wait_for_volume_resource_status(
                    self.os_admin.snapshots_v2_client,
                    snap['id'], 'available')

        # Get a given CG snapshot
        cgsnapshot = self.admin_consistencygroups_client.show_cgsnapshot(
            cgsnapshot['id'])['cgsnapshot']
        self.assertEqual(cgsnapshot_name, cgsnapshot['name'])

        # Get all CG snapshots with detail
        cgsnapshots = self.admin_consistencygroups_client.list_cgsnapshots(
            detail=True)['cgsnapshots']
        self.assertIn((cgsnapshot['name'], cgsnapshot['id']),
                      [(m['name'], m['id']) for m in cgsnapshots])

        # Clean up
        self._delete_cgsnapshot(cgsnapshot['id'], cg['id'])
        self._delete_consistencygroup(cg['id'])
        self.admin_volume_types_client.delete_volume_type(volume_type['id'])

    @decorators.idempotent_id('3a6a5525-25ca-4a6c-aac4-cac6fa8f5b43')
    def test_create_consistencygroup_from_cgsnapshot(self):
        # Create volume type
        name = data_utils.rand_name("volume-type")
        volume_type = self.admin_volume_types_client.create_volume_type(
            name=name)['volume_type']

        # Create CG
        cg_name = data_utils.rand_name('CG')
        create_consistencygroup = (
            self.admin_consistencygroups_client.create_consistencygroup)
        cg = create_consistencygroup(volume_type['id'],
                                     name=cg_name)['consistencygroup']
        self.admin_consistencygroups_client.wait_for_consistencygroup_status(
            cg['id'], 'available')
        self.assertEqual(cg_name, cg['name'])

        # Create volume
        vol_name = data_utils.rand_name("volume")
        params = {'name': vol_name,
                  'volume_type': volume_type['id'],
                  'consistencygroup_id': cg['id'],
                  'size': CONF.volume.volume_size}
        volume = self.admin_volume_client.create_volume(**params)['volume']
        waiters.wait_for_volume_resource_status(self.admin_volume_client,
                                                volume['id'], 'available')

        # Create cgsnapshot
        cgsnapshot_name = data_utils.rand_name('cgsnapshot')
        create_cgsnapshot = (
            self.admin_consistencygroups_client.create_cgsnapshot)
        cgsnapshot = create_cgsnapshot(cg['id'],
                                       name=cgsnapshot_name)['cgsnapshot']
        self.admin_consistencygroups_client.wait_for_cgsnapshot_status(
            cgsnapshot['id'], 'available')
        self.assertEqual(cgsnapshot_name, cgsnapshot['name'])
        snapshots = self.snapshots_client.list_snapshots(
            detail=True)['snapshots']
        for snap in snapshots:
            if volume['id'] == snap['volume_id']:
                waiters.wait_for_volume_resource_status(
                    self.os_admin.snapshots_v2_client, snap['id'], 'available')

        # Create CG from CG snapshot
        cg_name2 = data_utils.rand_name('CG_from_snap')
        create_consistencygroup2 = (
            self.admin_consistencygroups_client.
            create_consistencygroup_from_src
        )
        cg2 = create_consistencygroup2(cgsnapshot_id=cgsnapshot['id'],
                                       name=cg_name2)['consistencygroup']
        self.admin_consistencygroups_client.wait_for_consistencygroup_status(
            cg2['id'], 'available')
        self.assertEqual(cg_name2, cg2['name'])
        vols = self.admin_volume_client.list_volumes(
            detail=True)['volumes']
        for vol in vols:
            if vol['consistencygroup_id'] == cg2['id']:
                waiters.wait_for_volume_resource_status(
                    self.admin_volume_client, vol['id'], 'available')

        # Clean up
        self._delete_consistencygroup(cg2['id'])
        self._delete_cgsnapshot(cgsnapshot['id'], cg['id'])
        self._delete_consistencygroup(cg['id'])
        self.admin_volume_types_client.delete_volume_type(volume_type['id'])

    @decorators.idempotent_id('556121ae-de9c-4342-9897-e54260447a19')
    def test_create_consistencygroup_from_consistencygroup(self):
        # Create volume type
        name = data_utils.rand_name("volume-type")
        volume_type = self.admin_volume_types_client.create_volume_type(
            name=name)['volume_type']

        # Create CG
        cg_name = data_utils.rand_name('CG')
        create_consistencygroup = (
            self.admin_consistencygroups_client.create_consistencygroup)
        cg = create_consistencygroup(volume_type['id'],
                                     name=cg_name)['consistencygroup']
        self.admin_consistencygroups_client.wait_for_consistencygroup_status(
            cg['id'], 'available')
        self.assertEqual(cg_name, cg['name'])

        # Create volume
        vol_name = data_utils.rand_name("volume")
        params = {'name': vol_name,
                  'volume_type': volume_type['id'],
                  'consistencygroup_id': cg['id'],
                  'size': CONF.volume.volume_size}
        volume = self.admin_volume_client.create_volume(**params)['volume']
        waiters.wait_for_volume_resource_status(self.admin_volume_client,
                                                volume['id'], 'available')

        # Create CG from CG
        cg_name2 = data_utils.rand_name('CG_from_cg')
        create_consistencygroup2 = (
            self.admin_consistencygroups_client.
            create_consistencygroup_from_src
        )
        cg2 = create_consistencygroup2(source_cgid=cg['id'],
                                       name=cg_name2)['consistencygroup']
        self.admin_consistencygroups_client.wait_for_consistencygroup_status(
            cg2['id'], 'available')
        self.assertEqual(cg_name2, cg2['name'])
        vols = self.admin_volume_client.list_volumes(
            detail=True)['volumes']
        for vol in vols:
            if vol['consistencygroup_id'] == cg2['id']:
                waiters.wait_for_volume_resource_status(
                    self.admin_volume_client, vol['id'], 'available')

        # Clean up
        self._delete_consistencygroup(cg2['id'])
        self._delete_consistencygroup(cg['id'])
        self.admin_volume_types_client.delete_volume_type(volume_type['id'])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions

from cinder_tempest_plugin.rbac.v3 import base as rbac_base


class RbacV3VolumeTypesTests(rbac_base.VolumeV3RbacBaseTests):

    min_microversion = '3.3'
    extra_specs = {
        'key1': 'value1',
        'multiattach': '<is> False',
        'volume_backend_name': 'test-backend-name',
        'RESKEY:availability_zones': 'test-az',
        'replication_enabled': '<is> False'
    }
    extra_specs_keys = list(extra_specs.keys())
    expected_extra_specs = {
        "reader": [
            'multiattach', 'RESKEY:availability_zones', 'replication_enabled'
        ],
        "member": [
            'multiattach', 'RESKEY:availability_zones', 'replication_enabled'
        ],
        "admin": extra_specs_keys
    }

    encryption_type_key_cipher = 'cipher'
    create_kwargs = {
        'provider': 'LuksEncryptor',
        'key_size': 256,
        encryption_type_key_cipher: 'aes-xts-plain64',
        'control_location': 'front-end'
    }

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        admin_client = cls.os_project_admin
        cls.admin_volumes_client = admin_client.volumes_client_latest
        cls.admin_types_client = admin_client.volume_types_client_latest
        cls.admin_encryption_types_client = \
            admin_client.encryption_types_client_latest

    @classmethod
    def resource_setup(cls):
        """Create a new volume-type for the test"""
        super(RbacV3VolumeTypesTests, cls).resource_setup()
        # create a volume type
        cls.volume_type = cls.create_volume_type()

    @classmethod
    def create_volume_type(
            cls, name=None, with_encryption=True, cleanup=True
    ):
        # create a volume type
        if not name:
            name = data_utils.rand_name("volume-type")
        params = {'name': name,
                  'description': "description",
                  'extra_specs': cls.extra_specs,
                  'os-volume-type-access:is_public': True}
        volume_type = cls.admin_types_client.create_volume_type(
            **params
        )['volume_type']

        if with_encryption:
            # Create encryption_type
            cls.encryption_type = \
                cls.admin_encryption_types_client.create_encryption_type(
                    volume_type['id'], **cls.create_kwargs)['encryption']
            # NOTE: strictly speaking, this is NOT a volume_type field;
            # we save it for convenience in these tests
            volume_type['encryption_id'] = cls.encryption_type['encryption_id']

        if cleanup:
            cls.addClassResourceCleanup(
                cls.admin_types_client.delete_volume_type, volume_type['id']
            )

        return volume_type

    def _extra_specs_content_validator(self, client, extra_specs):
        """Validation of volume type's extra specs content

        Addition for feature:
        https://specs.openstack.org/openstack/cinder-specs/specs/xena/
        expose-cinder-user-visible-extra-specs-spec.html

        This feature allows 'readers' and 'members' to "see" volume type's
        extra specs:
        'multiattach', 'RESKEY:availability_zones' and 'replication_enabled'

        Args:
            client: Client object to be used
            extra_specs: extra_specs dict from response

        Returns:
            Boolean: True if lists are equal, false otherwise
        """
        role = client.user.split('-')[-1]
        return (sorted(list(extra_specs.keys())) ==
                sorted(self.expected_extra_specs[role]))

    def _update_volume_type(self, expected_status):
        """Update volume type"""
        self.do_request(
            method='update_volume_type',
            expected_status=expected_status,
            volume_type_id=self.volume_type['id'],
            description='Updated volume type description'
        )

    def _create_or_update_extra_specs_for_volume_type(self, expected_status):
        """Create or update extra specs"""
        volume_type = self.create_volume_type(with_encryption=False)
        # Create extra spec 'key2' with value 'value2'
        extra_spec = {'key2': 'value2'}
        self.do_request(
            method='create_volume_type_extra_specs',
            expected_status=expected_status,
            volume_type_id=volume_type['id'],
            extra_specs=extra_spec
        )

        # Update extra spec 'key2' with value 'updated value'
        extra_spec = {'key2': 'updated value'}
        self.do_request(
            method='update_volume_type_extra_specs',
            expected_status=expected_status,
            volume_type_id=volume_type['id'],
            extra_spec_name='key2',
            extra_specs=extra_spec
        )

    def _list_all_extra_specs_for_volume_type(self, expected_status):
        """List all extra_specs for a volume type"""
        extra_specs = self.do_request(
            method='list_volume_types_extra_specs',
            expected_status=expected_status,
            volume_type_id=self.volume_type['id']
        )['extra_specs']
        self.assertTrue(
            self._extra_specs_content_validator(
                client=self.client, extra_specs=extra_specs
            )
        )

    def _show_extra_spec_for_volume_type(self, expected_status):
        """Show extra_spec for a volume type"""

        # Using 'multiattach' extra spec because all admin, member and readers
        # should be able to "see".
        spec = self.do_request(
            method='show_volume_type_extra_specs',
            expected_status=expected_status,
            volume_type_id=self.volume_type['id'],
            extra_specs_name='multiattach'
        )
        self.assertEqual(spec['multiattach'], self.extra_specs['multiattach'])

        # Using 'volume_backend_name' extra spec because only admin should
        # "see" it.
        role = self.client.user.split('-')[-1]
        # 'reader' and 'member' will get 404 (NotFound) if they try to show
        # the extra spec 'volume_backend_name'
        try:
            spec = self.do_request(
                method='show_volume_type_extra_specs',
                expected_status=expected_status,
                volume_type_id=self.volume_type['id'],
                extra_specs_name='volume_backend_name'
            )
        except exceptions.NotFound:
            # NotFound exception should be thrown for
            # 'reader' and 'member' only
            self.assertNotEqual(
                role, 'admin',
                "NotFound exception was thrown for admin"
            )
            return

        # If no exception thrown, then check the content
        # Only admin should reach to this point
        self.assertNotIn(
            role, ['reader', 'member'],
            "NotFound should be thrown for non admin role"
        )
        self.assertEqual(
            spec['volume_backend_name'],
            self.extra_specs['volume_backend_name']
        )

    def _update_extra_spec_for_volume_type(self, expected_status):
        """Update extra_spec for a volume type"""
        extra_spec = {'key1': 'key1 updated value'}
        self.do_request(
            method='update_volume_type_extra_specs',
            expected_status=expected_status,
            volume_type_id=self.volume_type['id'],
            extra_spec_name='key1',
            extra_specs=extra_spec
        )

    def _delete_extra_spec_for_volume_type(self, expected_status):
        """Delete a volume type extra_spec"""
        volume_type = self.create_volume_type(with_encryption=False)

        self.do_request(
            method='delete_volume_type_extra_specs',
            expected_status=expected_status,
            volume_type_id=volume_type['id'],
            extra_spec_name='key1'
        )

    def _show_volume_type_detail(self, expected_status):
        """Show volume type"""
        details = self.do_request(
            method='show_volume_type',
            expected_status=expected_status,
            volume_type_id=self.volume_type['id']
        )['volume_type']
        self.assertTrue(
            self._extra_specs_content_validator(
                client=self.client, extra_specs=details['extra_specs']
            )
        )

    def _show_default_volume_type(self, expected_status):
        """Show default volume type"""
        self.do_request(
            method='show_default_volume_type',
            expected_status=expected_status
        )

    def _delete_volume_type(self, expected_status):
        """Delete a volume type"""
        cleanup = True if expected_status == exceptions.Forbidden\
            else False
        volume_type = self.create_volume_type(
            with_encryption=False, cleanup=cleanup
        )

        self.do_request(
            method='delete_volume_type',
            expected_status=expected_status,
            volume_type_id=volume_type['id']
        )

    def _list_volume_types(self, expected_status):
        """List all volume types"""
        volume_types = self.do_request(
            method='list_volume_types',
            expected_status=expected_status
        )['volume_types']
        for volume_type in volume_types:
            if volume_type['id'] == self.volume_type['id']:
                self._extra_specs_content_validator(
                    client=self.client, extra_specs=volume_type['extra_specs']
                )

    def _create_volume_type(self, expected_status):
        """Create a volume type"""
        volume_type = self.do_request(
            method='create_volume_type',
            expected_status=expected_status,
            name="test-new-volume-type"
        )
        if expected_status != exceptions.Forbidden:
            volume_type = volume_type['volume_type']
            self.admin_types_client.delete_volume_type(
                volume_type_id=volume_type['id']
            )

    def _show_encryption_type(self, expected_status):
        """Show volume type's encryption type"""
        self.do_request(
            method='show_encryption_type',
            expected_status=expected_status,
            client=self.encryption_types_client,
            volume_type_id=self.volume_type['id']
        )

    def _show_encryption_spec_item(self, expected_status):
        """Show encryption spec item"""
        self.do_request(
            method='show_encryption_specs_item',
            expected_status=expected_status,
            client=self.encryption_types_client,
            volume_type_id=self.volume_type['id'],
            key=self.encryption_type_key_cipher
        )

    def _delete_encryption_type(self, expected_status):
        """Delete encryption type"""
        volume_type = self.create_volume_type(with_encryption=True)

        self.do_request(
            method='delete_encryption_type',
            expected_status=expected_status,
            client=self.encryption_types_client,
            volume_type_id=volume_type['id'],
            encryption_id=volume_type['encryption_id']
        )

    def _create_encryption_type(self, expected_status):
        """Create encryption type"""
        volume_type = self.create_volume_type(with_encryption=False)

        self.do_request(
            method='create_encryption_type',
            expected_status=expected_status,
            client=self.encryption_types_client,
            volume_type_id=volume_type['id'],
            **self.create_kwargs
        )

    def _update_encryption_type(self, expected_status):
        """Update encryption type"""
        update_kwargs = {'key_size': 128}

        self.do_request(
            method='update_encryption_type',
            expected_status=expected_status,
            client=self.encryption_types_client,
            volume_type_id=self.volume_type['id'],
            encryption_id=self.volume_type['encryption_id'],
            **update_kwargs
        )


class VolumeTypesReaderTests(RbacV3VolumeTypesTests):
    """Test Volume types using 'reader' user"""
    credentials = ['project_reader', 'project_admin']

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_reader.volume_types_client_latest
        cls.encryption_types_client = \
            cls.os_project_reader.encryption_types_client_latest

    @decorators.idempotent_id('e3fdabf0-fd8c-4bab-9870-5a67fe25c6e4')
    def test_update_volume_type(self):
        self._update_volume_type(expected_status=exceptions.Forbidden)

    @decorators.idempotent_id('b046a4d7-79a0-436b-9075-863e2299b73d')
    def test_create_or_update_extra_specs_for_volume_type(self):
        self._create_or_update_extra_specs_for_volume_type(
            expected_status=exceptions.Forbidden
        )

    @decorators.idempotent_id('9499752c-3b27-41a3-8f55-4bdba7297f92')
    def test_list_all_extra_specs_for_volume_type(self):
        self._list_all_extra_specs_for_volume_type(
            expected_status=200
        )

    @decorators.idempotent_id('a38f7248-3a5b-4e51-8e32-d2dcf9c771ea')
    def test_show_extra_spec_for_volume_type(self):
        self._show_extra_spec_for_volume_type(expected_status=200)

    @decorators.idempotent_id('68689644-22a8-4ba6-a642-db4258681586')
    def test_update_extra_spec_for_volume_type(self):
        self._update_extra_spec_for_volume_type(
            expected_status=exceptions.Forbidden
        )

    @decorators.idempotent_id('a7cdd9ae-f389-48f6-b144-abf336b1637b')
    def test_delete_extra_spec_for_volume_type(self):
        self._delete_extra_spec_for_volume_type(
            expected_status=exceptions.Forbidden
        )

    @decorators.skip_because(bug='2016402')
    @decorators.idempotent_id('7ea28fc2-ce5a-48c9-8d03-31c2826fe566')
    def test_show_volume_type_detail(self):
        self._show_volume_type_detail(expected_status=200)

    @decorators.skip_because(bug='2016402')
    @decorators.idempotent_id('aceab52a-c503-4081-936e-b9df1c31046d')
    def test_show_default_volume_type(self):
        self._show_default_volume_type(expected_status=200)

    @decorators.idempotent_id('35581811-6288-4698-aaaf-7f5a4fe662e8')
    def test_delete_volume_type(self):
        self._delete_volume_type(expected_status=exceptions.Forbidden)

    @decorators.skip_because(bug='2016402')
    @decorators.idempotent_id('e8a438f9-e9c1-4f3f-8ae3-ad80ee02cd6a')
    def test_list_volume_types(self):
        self._list_volume_types(expected_status=200)

    @decorators.idempotent_id('3c3a39b1-fff5-492b-8c1c-9520063901ef')
    def test_create_volume_type(self):
        self._create_volume_type(expected_status=exceptions.Forbidden)

    @decorators.idempotent_id('84bd20f1-621c-416d-add2-fbae57137239')
    def test_show_encryption_type(self):
        self._show_encryption_type(expected_status=exceptions.Forbidden)

    @decorators.idempotent_id('ab9c7149-fab7-4584-b4ff-8b997cd62e75')
    def test_show_encryption_spec_item(self):
        self._show_encryption_spec_item(expected_status=exceptions.Forbidden)

    @decorators.idempotent_id('8d85ec39-bc32-4f49-88e6-63adc7e1f832')
    def test_delete_encryption_type(self):
        self._delete_encryption_type(expected_status=exceptions.Forbidden)

    @decorators.idempotent_id('c7c0892e-08d1-45e0-8ebf-be949cb4ab02')
    def test_create_encryption_type(self):
        self._create_encryption_type(expected_status=exceptions.Forbidden)

    @decorators.idempotent_id('8186d5bc-183a-4fcc-9c6a-e2b247a0caee')
    def test_update_encryption_type(self):
        self._update_encryption_type(expected_status=exceptions.Forbidden)


class VolumeTypesMemberTests(RbacV3VolumeTypesTests):
    """Test Volume types using 'member' user"""
    credentials = ['project_member', 'project_admin']

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_member.volume_types_client_latest
        cls.encryption_types_client = \
            cls.os_project_member.encryption_types_client_latest

    @decorators.idempotent_id('e5e642bf-2f31-4d04-ad43-6ad75562b7e4')
    def test_update_volume_type(self):
        self._update_volume_type(expected_status=exceptions.Forbidden)

    @decorators.idempotent_id('fda21e7e-9292-49b8-9754-f3c25b8e5f57')
    def test_create_or_update_extra_specs_for_volume_type(self):
        self._create_or_update_extra_specs_for_volume_type(
            expected_status=exceptions.Forbidden
        )

    @decorators.idempotent_id('82fd0d34-17b3-4f45-bd2e-728c9a8bff8c')
    def test_list_all_extra_specs_for_volume_type(self):
        self._list_all_extra_specs_for_volume_type(
            expected_status=200
        )

    @decorators.idempotent_id('67aa0b40-7c0a-4ae7-8682-fb4f20abd390')
    def test_show_extra_spec_for_volume_type(self):
        self._show_extra_spec_for_volume_type(expected_status=200)

    @decorators.idempotent_id('65470a71-254d-4152-bdaa-6b7f43e9c74f')
    def test_update_extra_spec_for_volume_type(self):
        self._update_extra_spec_for_volume_type(
            expected_status=exceptions.Forbidden
        )

    @decorators.idempotent_id('3695be33-bd22-4090-8252-9c42eb7eeef6')
    def test_delete_extra_spec_for_volume_type(self):
        self._delete_extra_spec_for_volume_type(
            expected_status=exceptions.Forbidden
        )

    @decorators.idempotent_id('319f3ca1-bdd7-433c-9bed-03c7b093e7a2')
    def test_show_volume_type_detail(self):
        self._show_volume_type_detail(expected_status=200)

    @decorators.skip_because(bug='2016402')
    @decorators.idempotent_id('2e990c61-a2ea-4a01-a2dc-1f483c934e8d')
    def test_show_default_volume_type(self):
        self._show_default_volume_type(expected_status=200)

    @decorators.idempotent_id('6847c211-647b-4d02-910c-773e76b99fcd')
    def test_delete_volume_type(self):
        self._delete_volume_type(expected_status=exceptions.Forbidden)

    @decorators.idempotent_id('308f80c9-6342-45a1-8e6e-9e400b510013')
    def test_list_volume_types(self):
        self._list_volume_types(expected_status=200)

    @decorators.idempotent_id('81cebbb8-fa0d-4bd8-a433-e43c7b187456')
    def test_create_volume_type(self):
        self._create_volume_type(expected_status=exceptions.Forbidden)

    @decorators.idempotent_id('7c84b013-c5a8-434f-8ea7-23c5b2d46d5e')
    def test_show_encryption_type(self):
        self._show_encryption_type(expected_status=exceptions.Forbidden)

    @decorators.idempotent_id('387974ce-3544-48e3-81c0-3f86a5b60b93')
    def test_show_encryption_spec_item(self):
        self._show_encryption_spec_item(expected_status=exceptions.Forbidden)

    @decorators.idempotent_id('c0163522-524f-4dfb-a3d4-6648f58ce99c')
    def test_delete_encryption_type(self):
        self._delete_encryption_type(expected_status=exceptions.Forbidden)

    @decorators.idempotent_id('65d86181-905a-4aa6-a9e5-672415d819a0')
    def test_create_encryption_type(self):
        self._create_encryption_type(expected_status=exceptions.Forbidden)

    @decorators.idempotent_id('2633f1d3-e648-4d12-86b9-e7f72b41ec68')
    def test_update_encryption_type(self):
        self._update_encryption_type(expected_status=exceptions.Forbidden)


class VolumeTypesAdminTests(RbacV3VolumeTypesTests):
    """Test Volume types using 'admin' user"""
    credentials = ['project_admin']

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_admin.volume_types_client_latest
        cls.encryption_types_client = \
            cls.os_project_admin.encryption_types_client_latest

    @decorators.idempotent_id('77d065ef-ffdd-4749-b326-d64fbf5d0432')
    def test_update_volume_type(self):
        self._update_volume_type(expected_status=200)

    @decorators.idempotent_id('422271a7-0128-4fd6-9f60-aeb4a1ce16ea')
    def test_create_or_update_extra_specs_for_volume_type(self):
        self._create_or_update_extra_specs_for_volume_type(
            expected_status=200
        )

    @decorators.idempotent_id('5c491d13-df15-4721-812e-2ed473b86a12')
    def test_list_all_extra_specs_for_volume_type(self):
        self._list_all_extra_specs_for_volume_type(
            expected_status=200
        )

    @decorators.idempotent_id('a2cca7b6-0af9-47e5-b8c1-4e0f01822d4e')
    def test_show_extra_spec_for_volume_type(self):
        self._show_extra_spec_for_volume_type(expected_status=200)

    @decorators.idempotent_id('d0ff17d3-2c47-485f-b2f1-d53ec32c32e2')
    def test_update_extra_spec_for_volume_type(self):
        self._update_extra_spec_for_volume_type(
            expected_status=200
        )

    @decorators.idempotent_id('4661cc2f-8727-4998-a427-8cb1d512b68a')
    def test_delete_extra_spec_for_volume_type(self):
        self._delete_extra_spec_for_volume_type(
            expected_status=202
        )

    @decorators.idempotent_id('7f794e33-b5cf-4172-b39e-a56cd9c18a2e')
    def test_show_volume_type_detail(self):
        self._show_volume_type_detail(expected_status=200)

    @decorators.skip_because(bug='2016402')
    @decorators.idempotent_id('93886ad8-5cd0-4def-8b0e-40418e55050d')
    def test_show_default_volume_type(self):
        self._show_default_volume_type(expected_status=200)

    @decorators.idempotent_id('7486259d-5c40-4fb3-8a95-491c45a0a872')
    def test_delete_volume_type(self):
        self._delete_volume_type(expected_status=202)

    @decorators.idempotent_id('e075e8ff-bb05-4c84-b2ab-0205ef3e8dbd')
    def test_list_volume_types(self):
        self._list_volume_types(expected_status=200)

    @decorators.idempotent_id('57384db2-9408-4a31-8c15-022eea5f9b76')
    def test_create_volume_type(self):
        self._create_volume_type(expected_status=200)

    @decorators.idempotent_id('46fc49a3-f76f-4c22-ac83-8d1665437810')
    def test_show_encryption_type(self):
        self._show_encryption_type(expected_status=200)

    @decorators.idempotent_id('4ff57649-bfe1-48f4-aaac-4577affba8d7')
    def test_show_encryption_spec_item(self):
        self._show_encryption_spec_item(expected_status=200)

    @decorators.idempotent_id('e622af7d-a412-4903-9256-256d8e3cc560')
    def test_delete_encryption_type(self):
        self._delete_encryption_type(expected_status=202)

    @decorators.idempotent_id('e7c4e925-6ce6-439b-8be8-6df4cbc32cdc')
    def test_create_encryption_type(self):
        self._create_encryption_type(expected_status=200)

    @decorators.idempotent_id('90beb71d-93fa-4252-8566-192bdd517715')
    def test_update_encryption_type(self):
        self._update_encryption_type(expected_status=200)



# Copyright 2021 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import contextlib

from oslo_log import log

from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import exceptions as lib_exc

from tempest.scenario import manager

CONF = config.CONF

LOG = log.getLogger(__name__)


class ScenarioTest(manager.ScenarioTest):

    credentials = ['primary', 'admin']

    @classmethod
    def setup_clients(cls):
        super(ScenarioTest, cls).setup_clients()
        cls.admin_volume_types_client = cls.os_admin.volume_types_client_latest

    def _attached_volume_name(
            self, disks_list_before_attach, ip_address, private_key):
        ssh = self.get_remote_client(ip_address, private_key=private_key)

        def _wait_for_volume_available_on_system():
            disks_list_after_attach = ssh.list_disks()
            return len(disks_list_after_attach) > len(disks_list_before_attach)

        if not test_utils.call_until_true(_wait_for_volume_available_on_system,
                                          CONF.compute.build_timeout,
                                          CONF.compute.build_interval):
            raise lib_exc.TimeoutException

        disks_list_after_attach = ssh.list_disks()
        volume_name = [item for item in disks_list_after_attach
                       if item not in disks_list_before_attach][0]
        return volume_name

    @contextlib.contextmanager
    def mount_dev_path(self, ssh_client, dev_name, mount_path):
        if dev_name is not None:
            ssh_client.exec_command('sudo mount /dev/%s %s' % (dev_name,
                                                               mount_path))
            yield
            ssh_client.exec_command('sudo umount %s' % mount_path)
        else:
            yield

    def _get_file_md5(self, ip_address, filename, dev_name=None,
                      mount_path='/mnt', private_key=None, server=None):

        ssh_client = self.get_remote_client(ip_address,
                                            private_key=private_key,
                                            server=server)
        with self.mount_dev_path(ssh_client, dev_name, mount_path):
            md5_sum = ssh_client.exec_command(
                'sudo md5sum %s/%s|cut -c 1-32' % (mount_path, filename))
        return md5_sum

    def _count_files(self, ip_address, dev_name=None, mount_path='/mnt',
                     private_key=None, server=None):
        ssh_client = self.get_remote_client(ip_address,
                                            private_key=private_key,
                                            server=server)
        with self.mount_dev_path(ssh_client, dev_name, mount_path):
            count = ssh_client.exec_command(
                'sudo ls -l %s | wc -l' % mount_path)
        # We subtract 2 from the count since `wc -l` also includes the count
        # of new line character and while creating the filesystem, a
        # lost+found folder is also created
        return int(count) - 2

    def _make_fs(self, ip_address, private_key, server, dev_name, fs='ext4'):
        ssh_client = self.get_remote_client(ip_address,
                                            private_key=private_key,
                                            server=server)

        ssh_client.make_fs(dev_name, fs=fs)

    def create_md5_new_file(self, ip_address, filename, dev_name=None,
                            mount_path='/mnt', private_key=None, server=None):
        ssh_client = self.get_remote_client(ip_address,
                                            private_key=private_key,
                                            server=server)

        with self.mount_dev_path(ssh_client, dev_name, mount_path):
            ssh_client.exec_command(
                'sudo dd bs=1024 count=100 if=/dev/urandom of=/%s/%s' %
                (mount_path, filename))
            md5 = ssh_client.exec_command(
                'sudo md5sum -b %s/%s|cut -c 1-32' % (mount_path, filename))
        return md5

    def get_md5_from_file(self, instance, instance_ip, filename,
                          dev_name=None):

        md5_sum = self._get_file_md5(instance_ip, filename=filename,
                                     dev_name=dev_name,
                                     private_key=self.keypair['private_key'],
                                     server=instance)
        count = self._count_files(instance_ip, dev_name=dev_name,
                                  private_key=self.keypair['private_key'],
                                  server=instance)
        return count, md5_sum

    def write_data_to_device(self, ip_address, out_dev, in_dev='/dev/urandom',
                             bs=1024, count=100, private_key=None,
                             server=None, sha_sum=False):
        ssh_client = self.get_remote_client(
            ip_address, private_key=private_key, server=server)

        # Write data to device
        write_command = (
            'sudo dd bs=%(bs)s count=%(count)s if=%(in_dev)s of=%(out_dev)s '
            '&& sudo dd bs=%(bs)s count=%(count)s if=%(out_dev)s' %
            {'bs': str(bs), 'count': str(count), 'in_dev': in_dev,
             'out_dev': out_dev})
        if sha_sum:
            # If we want to read sha1sum instead of the device data
            write_command += ' | sha1sum | head -c 40'
        data = ssh_client.exec_command(write_command)

        return data

    def read_data_from_device(self, ip_address, in_dev, bs=1024, count=100,
                              private_key=None, server=None, sha_sum=False):
        ssh_client = self.get_remote_client(
            ip_address, private_key=private_key, server=server)

        # Read data from device
        read_command = ('sudo dd bs=%(bs)s count=%(count)s if=%(in_dev)s' %
                        {'bs': bs, 'count': count, 'in_dev': in_dev})
        if sha_sum:
            # If we want to read sha1sum instead of the device data
            read_command += ' | sha1sum  | head -c 40'
        data = ssh_client.exec_command(read_command)

        return data

    def _attach_and_get_volume_device_name(self, server, volume, instance_ip,
                                           private_key):
        ssh_client = self.get_remote_client(
            instance_ip, private_key=private_key,
            server=server)
        # List disks before volume attachment
        disks_list_before_attach = ssh_client.list_disks()
        # Attach volume
        attachment = self.attach_volume(server, volume)
        # Find the difference between disks before and after attachment that
        # gives us the volume device name
        volume_device_name = self._attached_volume_name(
            disks_list_before_attach, instance_ip, private_key)
        return volume_device_name, attachment

    def create_volume_type(self, client=None, name=None, extra_specs=None):
        if not client:
            client = self.os_admin.volume_types_client_latest
        if not name:
            class_name = self.__class__.__name__
            name = data_utils.rand_name(class_name + '-volume-type')
        randomized_name = data_utils.rand_name('scenario-type-' + name)

        LOG.debug("Creating a volume type: %s with extra_specs %s",
                  randomized_name, extra_specs)
        if extra_specs is None:
            extra_specs = {}
        volume_type = self.admin_volume_types_client.create_volume_type(
            name=randomized_name, extra_specs=extra_specs)['volume_type']
        self.addCleanup(self.cleanup_volume_type, volume_type)
        return volume_type

    def attach_volume(self, server, volume, device=None, tag=None):
        """Attaches volume to server and waits for 'in-use' volume status.

        The volume will be detached when the test tears down.

        :param server: The server to which the volume will be attached.
        :param volume: The volume to attach.
        :param device: Optional mountpoint for the attached volume. Note that
            this is not guaranteed for all hypervisors and is not recommended.
        :param tag: Optional device role tag to apply to the volume.
        """
        attach_kwargs = dict(volumeId=volume['id'])
        if device:
            attach_kwargs['device'] = device
        if tag:
            attach_kwargs['tag'] = tag

        attachment = self.servers_client.attach_volume(
            server['id'], **attach_kwargs)['volumeAttachment']
        # On teardown detach the volume and for multiattach volumes wait for
        # the attachment to be removed. For non-multiattach volumes wait for
        # the state of the volume to change to available. This is so we don't
        # error out when trying to delete the volume during teardown.
        if volume['multiattach']:
            att = waiters.wait_for_volume_attachment_create(
                self.volumes_client, volume['id'], server['id'])
            self.addCleanup(waiters.wait_for_volume_attachment_remove,
                            self.volumes_client, volume['id'],
                            att['attachment_id'])
        else:
            self.addCleanup(waiters.wait_for_volume_resource_status,
                            self.volumes_client, volume['id'], 'available')
            waiters.wait_for_volume_resource_status(self.volumes_client,
                                                    volume['id'], 'in-use')
        # Ignore 404s on detach in case the server is deleted or the volume
        # is already detached.
        self.addCleanup(self._detach_volume, server, volume)
        return attachment

    def _detach_volume(self, server, volume):
        """Helper method to detach a volume.

        Ignores 404 responses if the volume or server do not exist, or the
        volume is already detached from the server.
        """
        try:
            volume = self.volumes_client.show_volume(volume['id'])['volume']
            # Check the status. You can only detach an in-use volume, otherwise
            # the compute API will return a 400 response.
            if volume['status'] == 'in-use':
                self.servers_client.detach_volume(server['id'], volume['id'])
        except lib_exc.NotFound:
            # Ignore 404s on detach in case the server is deleted or the volume
            # is already detached.
            pass



# Copyright (C) 2017 Huawei.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json
from tempest.lib.common import rest_client
from tempest.lib.services.volume import base_client


class VolumeRevertClient(base_client.BaseClient):
    """Client class to send revert to snapshot action API request"""

    def __init__(self, auth_provider, service, region, **kwargs):
        super(VolumeRevertClient, self).__init__(
            auth_provider, service, region, **kwargs)
        # revert needs v3 of the volume API
        self.api_version = 'v3'

    def revert_to_snapshot(self, volume, snapshot_id):
        """Revert a volume to snapshot."""
        post_body = {'snapshot_id': snapshot_id}
        post_body = json.dumps({'revert': post_body})
        resp, body = self.post('volumes/%s/action' % volume['id'],
                               post_body)
        return rest_client.ResponseBody(resp, body)



# -*- coding: utf-8 -*-
# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
import testtools

from cinder_tempest_plugin.api.volume import base

CONF = config.CONF


class CinderUnicodeTest(base.BaseVolumeTest):

    @classmethod
    def resource_setup(cls):
        super(CinderUnicodeTest, cls).resource_setup()

        # Stick to three-byte unicode here, since four+ byte
        # chars require utf8mb4 database support which may not
        # be configured.
        cls.volume_name = u"CinderUnicodeTest塵㼗‽"
        cls.volume = cls.create_volume_with_args(name=cls.volume_name)

    @classmethod
    def create_volume_with_args(cls, **kwargs):
        if 'name' not in kwargs:
            kwargs['name'] = data_utils.rand_name('Volume')

        kwargs['size'] = CONF.volume.volume_size

        volume = cls.volumes_client.create_volume(**kwargs)['volume']
        cls.addClassResourceCleanup(
            cls.volumes_client.wait_for_resource_deletion, volume['id'])
        cls.addClassResourceCleanup(test_utils.call_and_ignore_notfound_exc,
                                    cls.volumes_client.delete_volume,
                                    volume['id'])
        waiters.wait_for_volume_resource_status(cls.volumes_client,
                                                volume['id'],
                                                'available')

        return volume

    @decorators.idempotent_id('2d7e2e49-150e-4849-a18e-79f9777c9a96')
    def test_create_delete_unicode_volume_name(self):
        """Create a volume with a unicode name and view it."""

        result = self.volumes_client.show_volume(self.volume['id'])
        fetched_volume = result['volume']
        self.assertEqual(fetched_volume['name'],
                         self.volume_name)

    @testtools.skipUnless(CONF.volume_feature_enabled.snapshot,
                          "Cinder volume snapshots are disabled")
    @decorators.related_bug('1393871')
    @decorators.idempotent_id('332be44d-5418-4fb3-a8f0-a3587de6929f')
    def test_snapshot_create_volume_description_non_ascii_code(self):
        # Create a volume with non-ascii description
        description = '\u05e7\u05d9\u05d9\u05e4\u05e9'
        volume = self.create_volume(description=description)
        vol_info = self.volumes_client.show_volume(volume['id'])['volume']
        self.assertEqual(description, vol_info['description'])

        # Create a snapshot with different non-ascii description
        description = '\u4e2d\u56fd\u793e\u533a'
        snapshot = self.create_snapshot(volume['id'], description=description)
        snapshot_info = self.snapshots_client.show_snapshot(
            snapshot['id'])['snapshot']
        self.assertEqual(description, snapshot_info['description'])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators

from cinder_tempest_plugin.api.volume import base

CONF = config.CONF


class VolumeAndVolumeTypeFromImageTest(base.BaseVolumeAdminTest):
    # needs AdminTest as superclass to manipulate volume_types

    @classmethod
    def skip_checks(cls):
        super(VolumeAndVolumeTypeFromImageTest, cls).skip_checks()
        if not CONF.service_available.glance:
            raise cls.skipException("Glance service is disabled")

    @decorators.idempotent_id('6e9266ff-a917-4dd5-aa4a-c36e59e7a2a6')
    def test_create_from_image_with_volume_type_image_property(self):
        """Verify that the cinder_img_volume_type image property works.

        When a volume is created from an image containing the
        cinder_img_volume_type property and no volume_type is specified
        in the volume-create request, the volume_type of the resulting
        volume should be the one specified by the image property.
        """

        volume_type_meta = 'cinder_img_volume_type'
        volume_type_name = 'vol-type-for-6e9266ff-a917-4dd5-aa4a-c36e59e7a2a6'
        description = ('Generic volume_type for test '
                       '6e9266ff-a917-4dd5-aa4a-c36e59e7a2a6')
        proto = CONF.volume.storage_protocol
        vendor = CONF.volume.vendor_name
        extra_specs = {"storage_protocol": proto,
                       "vendor_name": vendor}
        kwargs = {'description': description,
                  'extra_specs': extra_specs,
                  'os-volume-type-access:is_public': True}
        volume_type = self.create_volume_type(name=volume_type_name,
                                              **kwargs)
        # quick sanity check
        self.assertEqual(volume_type_name, volume_type['name'])

        # create an image in glance
        kwargs = {'disk_format': 'raw',
                  'container_format': 'bare',
                  'name': ('image-for-test-'
                           '6e9266ff-a917-4dd5-aa4a-c36e59e7a2a6'),
                  'visibility': 'private',
                  volume_type_meta: volume_type_name}
        image = self.create_image_with_data(**kwargs)
        # quick sanity check
        self.assertEqual(volume_type_name, image[volume_type_meta])

        # create volume from image
        kwargs = {'name': ('volume-for-test-'
                           '6e9266ff-a917-4dd5-aa4a-c36e59e7a2a6'),
                  'imageRef': image['id']}
        # this is the whole point of the test, so make sure this is true
        self.assertNotIn('volume_type', kwargs)
        volume = self.create_volume(**kwargs)

        found_volume_type = volume['volume_type']
        self.assertEqual(volume_type_name, found_volume_type)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

from tempest.scenario import manager

CONF = config.CONF


class TestEncryptedCinderVolumes(manager.EncryptionScenarioTest,
                                 manager.ScenarioTest):

    @classmethod
    def skip_checks(cls):
        super(TestEncryptedCinderVolumes, cls).skip_checks()
        if not CONF.compute_feature_enabled.attach_encrypted_volume:
            raise cls.skipException('Encrypted volume attach is not supported')

    @classmethod
    def resource_setup(cls):
        super(TestEncryptedCinderVolumes, cls).resource_setup()

    @classmethod
    def resource_cleanup(cls):
        super(TestEncryptedCinderVolumes, cls).resource_cleanup()

    def attach_detach_volume(self, server, volume):
        attached_volume = self.nova_volume_attach(server, volume)
        self.nova_volume_detach(server, attached_volume)

    def _delete_server(self, server):
        self.servers_client.delete_server(server['id'])
        waiters.wait_for_server_termination(self.servers_client, server['id'])

    def create_encrypted_volume_from_image(self, encryption_provider,
                                           volume_type='luks',
                                           key_size=256,
                                           cipher='aes-xts-plain64',
                                           control_location='front-end',
                                           **kwargs):
        """Create an encrypted volume from image.

        :param image_id: ID of the image to create volume from,
            CONF.compute.image_ref by default
        :param name: name of the volume,
            '$classname-volume-origin' by default
        :param **kwargs: additional parameters
        """
        volume_type = self.create_volume_type(name=volume_type)
        self.create_encryption_type(type_id=volume_type['id'],
                                    provider=encryption_provider,
                                    key_size=key_size,
                                    cipher=cipher,
                                    control_location=control_location)
        image_id = kwargs.pop('image_id', CONF.compute.image_ref)
        name = kwargs.pop('name', None)
        if not name:
            namestart = self.__class__.__name__ + '-volume-origin'
            name = data_utils.rand_name(namestart)
        return self.create_volume(volume_type=volume_type['name'],
                                  name=name, imageRef=image_id,
                                  **kwargs)

    @decorators.idempotent_id('5bb622ab-5060-48a8-8840-d589a548b9e4')
    @utils.services('volume')
    @utils.services('compute')
    def test_attach_cloned_encrypted_volume(self):

        """This test case attempts to reproduce the following steps:

        * Create an encrypted volume
        * Create clone from volume
        * Boot an instance and attach/dettach cloned volume

        """

        volume = self.create_encrypted_volume('luks', volume_type='luks')
        kwargs = {
            'display_name': data_utils.rand_name(self.__class__.__name__),
            'source_volid': volume['id'],
            'volume_type': volume['volume_type'],
            'size': volume['size']
        }
        volume_s = self.volumes_client.create_volume(**kwargs)['volume']
        self.addCleanup(self.volumes_client.wait_for_resource_deletion,
                        volume_s['id'])
        self.addCleanup(self.volumes_client.delete_volume, volume_s['id'])
        waiters.wait_for_volume_resource_status(
            self.volumes_client, volume_s['id'], 'available')
        volume_source = self.volumes_client.show_volume(
            volume_s['id'])['volume']
        validation_resources = self.get_test_validation_resources(
            self.os_primary)
        server = self.create_server(wait_until='SSHABLE',
                                    validatable=True,
                                    validation_resources=validation_resources)
        self.attach_detach_volume(server, volume_source)

    @decorators.idempotent_id('5bb622ab-5060-48a8-8840-d589a548b7e4')
    @utils.services('volume')
    @utils.services('compute')
    @utils.services('image')
    def test_boot_cloned_encrypted_volume(self):

        """This test case attempts to reproduce the following steps:

        * Create an encrypted volume from image
        * Boot an instance from the volume
        * Write data to the volume
        * Destroy the instance
        * Create a clone of the encrypted volume
        * Boot an instance from cloned volume
        * Verify the data
        """

        keypair = self.create_keypair()
        security_group = self.create_security_group()

        volume = self.create_encrypted_volume_from_image('luks')

        # create an instance from volume
        instance_1st = self.boot_instance_from_resource(
            source_id=volume['id'],
            source_type='volume',
            keypair=keypair,
            security_group=security_group)

        # write content to volume on instance
        ip_instance_1st = self.get_server_ip(instance_1st)
        timestamp = self.create_timestamp(ip_instance_1st,
                                          private_key=keypair['private_key'],
                                          server=instance_1st)
        # delete instance
        self._delete_server(instance_1st)

        # create clone
        kwargs = {
            'display_name': data_utils.rand_name(self.__class__.__name__),
            'source_volid': volume['id'],
            'volume_type': volume['volume_type'],
            'size': volume['size']
        }
        volume_s = self.volumes_client.create_volume(**kwargs)['volume']

        self.addCleanup(self.volumes_client.wait_for_resource_deletion,
                        volume_s['id'])
        self.addCleanup(self.volumes_client.delete_volume, volume_s['id'])
        waiters.wait_for_volume_resource_status(
            self.volumes_client, volume_s['id'], 'available')

        # create an instance from volume
        instance_2nd = self.boot_instance_from_resource(
            source_id=volume_s['id'],
            source_type='volume',
            keypair=keypair,
            security_group=security_group)

        # check the content of written file
        ip_instance_2nd = self.get_server_ip(instance_2nd)
        timestamp2 = self.get_timestamp(ip_instance_2nd,
                                        private_key=keypair['private_key'],
                                        server=instance_2nd)

        self.assertEqual(timestamp, timestamp2)

        # delete instance
        self._delete_server(instance_2nd)



# Copyright (c) 2017 Huawei.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.common import waiters
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions

from cinder_tempest_plugin.api.volume import base

CONF = config.CONF


class VolumeRevertTests(base.BaseVolumeTest):
    min_microversion = '3.40'

    @classmethod
    def skip_checks(cls):
        super(VolumeRevertTests, cls).skip_checks()
        if not CONF.volume_feature_enabled.volume_revert:
            raise cls.skipException("Cinder volume revert feature disabled")

    def setUp(self):
        super(VolumeRevertTests, self).setUp()
        # Create volume
        self.volume = self.create_volume()
        # Create snapshot
        self.snapshot = self.create_snapshot(self.volume['id'],
                                             metadata={'mykey1': 'value1'})

    @decorators.idempotent_id('87b7dcb7-4950-4a3a-802c-ece55491846d')
    def test_volume_revert_to_snapshot(self):
        """Test revert to snapshot"""
        expected_size = self.volume['size']
        # Revert to snapshot
        self.volume_revert_client.revert_to_snapshot(self.volume,
                                                     self.snapshot['id'])
        waiters.wait_for_volume_resource_status(
            self.volumes_client,
            self.volume['id'], 'available')
        waiters.wait_for_volume_resource_status(
            self.snapshots_client,
            self.snapshot['id'], 'available')
        volume = self.volumes_client.show_volume(self.volume['id'])['volume']

        self.assertEqual(expected_size, volume['size'])

    @decorators.idempotent_id('4e8b0788-87fe-430d-be7a-444d7f8e0347')
    def test_volume_revert_to_snapshot_after_extended_negative(self):
        """Test revert to snapshot after extended"""
        # Extend volume to double the size
        expected_size = self.volume['size'] * 2
        # Extend the volume
        self.volumes_client.extend_volume(self.volume['id'],
                                          new_size=expected_size)
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                self.volume['id'], 'available')

        # Destination volume smaller than source, API should block that
        self.assertRaises(exceptions.BadRequest,
                          self.volume_revert_client.revert_to_snapshot,
                          self.volume, self.snapshot['id'])



# Copyright (C) 2020 Canonical Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions

from cinder_tempest_plugin.api.volume import base

CONF = config.CONF


class VolumesBackupsTest(base.BaseVolumeAdminTest):
    @classmethod
    def skip_checks(cls):
        super(VolumesBackupsTest, cls).skip_checks()
        if not CONF.volume_feature_enabled.backup:
            raise cls.skipException("Cinder backup feature disabled")

    @decorators.idempotent_id('2daadb2e-409a-4ede-a6ce-6002ec324372')
    def test_backup_crossproject_admin_negative(self):

        # create vol as user
        volume = self.create_volume(size=CONF.volume.volume_size)

        # create backup as user
        self.create_backup(volume_id=volume['id'])
        # try to create incremental backup as admin
        self.assertRaises(
            exceptions.BadRequest, self.admin_backups_client.create_backup,
            volume_id=volume['id'], incremental=True)

    @decorators.idempotent_id('b9feb593-5809-4207-90d3-28e627730f13')
    def test_backup_crossproject_user_negative(self):

        # create vol as user
        volume = self.create_volume(size=CONF.volume.volume_size)

        # create backup as admin

        self.create_backup(volume_id=volume['id'],
                           backup_client=self.admin_backups_client)

        # try to create incremental backup as user
        self.assertRaises(
            exceptions.BadRequest, self.backups_client.create_backup,
            volume_id=volume['id'], incremental=True)

    @decorators.idempotent_id('ce15f528-bfc1-492d-81db-b6168b631587')
    def test_incremental_backup_respective_parents(self):

        # create vol as user
        volume = self.create_volume(size=CONF.volume.volume_size)

        # create backup as admin
        backup_adm = self.create_backup(
            volume_id=volume['id'], backup_client=self.admin_backups_client)

        # create backup as user
        backup_usr = self.create_backup(volume_id=volume['id'])

        # refresh admin backup and assert no child backups
        backup_adm = self.admin_backups_client.show_backup(
            backup_adm['id'])['backup']
        self.assertFalse(backup_adm['has_dependent_backups'])

        # create incremental backup as admin
        self.create_backup(volume_id=volume['id'], incremental=True,
                           backup_client=self.admin_backups_client)

        # refresh user backup and assert no child backups
        backup_usr = self.backups_client.show_backup(
            backup_usr['id'])['backup']
        self.assertFalse(backup_usr['has_dependent_backups'])

        # refresh admin backup and assert it has childs
        backup_adm = self.admin_backups_client.show_backup(
            backup_adm['id'])['backup']
        self.assertTrue(backup_adm['has_dependent_backups'])

        # create incremental backup as user
        self.create_backup(volume_id=volume['id'],
                           incremental=True)

        # refresh user backup and assert it has childs
        backup_usr = self.backups_client.show_backup(
            backup_usr['id'])['backup']
        self.assertTrue(backup_usr['has_dependent_backups'])
